import functools
import unittest
import xml.etree.ElementTree
import stubout

import yagi.config
import yagi.serializer
from yagi.serializer import atom

class SerializerTests(unittest.TestCase):
    def setUp(self):
        self.stubs = stubout.StubOutForTesting()
        self.test_event = {
            "event_type": "compute.instance.update",
            "timestamp": "2013-08-06 13:57:57.468516",
            "priority": "INFO",
            "publisher_id": "compute.test",
            "message_id": "46d55117-d2ff-4124-a184-5ff21a5c4c9b",
            "payload": {"state_description": "",
                        "availability_zone": None,
                        "ephemeral_gb": 0,
                        "instance_type_id": 6,
                        "bandwidth": {},
                        "deleted_at": "",
                        "reservation_id": "r-z2ckc2ds",
                        "memory_mb": 64,
                        "user_id": "2b0d6ab80bc6410e9694772cb344a309",
                        "hostname": "test2",
                        "state": "error",
                        "old_state": "error",
                        "old_task_state": "spawning",
                        "metadata": [],
                        "node": "compute.test",
                        "ramdisk_id": "01dff59f-a3eb-466c-b0c8-63712f586b06",
                        "access_ip_v6": None,
                        "access_ip_v4": None,
                        "disk_gb": 0,
                        "kernel_id": "61b7c758-9e9a-4381-bc24-70931e0f8786",
                        "host": "compute.test",
                        "display_name": "test2",
                        "image_ref_url": "http://127.0.0.1:9292/images/309019bb-742b-45f1-a809-e0be7c6123d0",
                        "audit_period_beginning": "2013-08-06T13:00:00.000000",
                        "audit_period_ending": "2013-08-06T13:57:57.465039",
                        "root_gb": 0,
                        "tenant_id": "71e7841cace54b80b1bc98af3cdfd7bc",
                        "created_at": "2013-08-06 13:57:49",
                        "launched_at": "",
                        "instance_id": "475283c0-2a2f-4ac1-b2c0-cc8e9a67310d",
                        "instance_type": "m1.nano",
                        "vcpus": 1,
                        "image_meta": {"kernel_id": "61b7c758-9e9a-4381-bc24-70931e0f8786",
                                       "ramdisk_id": "01dff59f-a3eb-466c-b0c8-63712f586b06",
                                       "base_image_ref": "309019bb-742b-45f1-a809-e0be7c6123d0"
                                       },
                        "architecture": None,
                        "new_task_state": None,
                        "os_type": None,
                        },
        }
        self.test_entity = dict(content=self.test_event,
                                id=self.test_event['message_id'],
                                event_type=self.test_event['event_type'])
        self.ns='{http://www.w3.org/2005/Atom}'

        config_dict = {
            'event_feed': {
                'feed_title': 'feed_title',
                'feed_host': 'feed_host',
                'use_https': False,
                'serializer_driver': 'yagi.serializer.atom',
                'port': 'port',
                'atom_categories': "Test, 123, The, Thing"
            },
        }

        def get(*args, **kwargs):
            val = None
            for arg in args:
                if val:
                    val = val.get(arg)
                else:
                    val = config_dict.get(arg)
                    if not val:
                        return None or kwargs.get('default')
            return val

        def config_with(*args):
            return functools.partial(get, args)

        self.stubs.Set(yagi.config, 'config_with', config_with)
        self.stubs.Set(yagi.config, 'get', get)

    def tearDown(self):
        self.stubs.UnsetAll()

    def test_no_link(self):
        body = atom.dump_item(self.test_entity, entity_links=False)
        xml_data = xml.etree.ElementTree.fromstring(body)
        links = xml_data.findall(self.ns+'link')
        self.assertEquals(len(links),0)

    def test_link(self):
        body = atom.dump_item(self.test_entity, entity_links=True)
        xml_data = xml.etree.ElementTree.fromstring(body)
        links = xml_data.findall(self.ns+'link')
        self.assertEquals(len(links),1)

    def test_entry_id(self):
        body = atom.dump_item(self.test_entity)
        xml_data = xml.etree.ElementTree.fromstring(body)
        ids = xml_data.findall(self.ns+'id')
        self.assertEquals(len(ids),1)
        id_tag = ids[0]
        self.assertEquals(id_tag.text, 'urn:uuid:46d55117-d2ff-4124-a184-5ff21a5c4c9b')

    def test_dump_item(self):
        body = atom.dump_item(self.test_entity)
        self.assertTrue(len(body) > 0)
        xml_data = xml.etree.ElementTree.fromstring(body)
        self.assertEquals(xml_data.tag, self.ns+'entry')

    def test_load_serializer(self):
        """Contrived test for basic functionality"""
        ser = yagi.serializer.feed_serializer()
        self.assertEqual(ser, yagi.serializer.atom)



import datetime
import socket
import time
import logging

import amqplib
from carrot.connection import BrokerConnection
from carrot.messaging import Consumer

from yagi import config as conf

with conf.defaults_for("global") as default:
    default("update_timer", 300)

with conf.defaults_for("rabbit_broker") as default:
    default("host", "localhost")
    default("user", "guest")
    default("password", "guest")
    default("port", 5672)
    default("vhost", "/")
    default("poll_delay", 1)
    default("reconnect_delay", 5)
    default("max_wait", 600)
    default("max_connection_age", 14400)

LOG = logging.getLogger(__name__)

def confbool(val):
    if val in ("Default", None, "None"):
        return None
    return val == "True" or False


class NotQuiteSoStupidConsumer(Consumer):
    """ Carrot is quite broken/braindead in a number of ways.
    The proper way we should fix this is by having Yagi use Kombu for it"s
    queue reading, however, until we have that done, this shim class fixes
    some of the carrot consumer class"s brokenness.

    To be specific: Both queues and exchanges can be declared as durable
    and/or auto_delete (or not), *separately*. Carrot uses the same setting
    for declaring both queues and exchange. THus if your queue is durable,
    and the exchange is not, or vice versa, carrot cannot handle it.
    I don"t know why you would want that, but Nova currently declares it"s
    queues this way."""

    _init_opts = Consumer._init_opts + ("exchange_durable",
                                        "exchange_auto_delete")

    exchange_durable = None
    exchange_auto_delete = None

    def declare(self):
        """Declares the queue, the exchange and binds the queue to
           the exchange."""
        arguments = None
        routing_key = self.routing_key
        if self.exchange_type == "headers":
            arguments, routing_key = routing_key, ""

        if self.queue:
            self.backend.queue_declare(queue=self.queue, durable=self.durable,
                                       exclusive=self.exclusive,
                                       auto_delete=self.auto_delete,
                                       warn_if_exists=self.warn_if_exists)
        edurable = self.durable \
                   if self.exchange_durable is None \
                   else self.exchange_durable
        eauto_delete = self.auto_delete \
                   if self.exchange_auto_delete is None \
                   else self.exchange_auto_delete

        if self.exchange:
            self.backend.exchange_declare(exchange=self.exchange,
                                          type=self.exchange_type,
                                          durable=edurable,
                                          auto_delete=eauto_delete)
        if self.queue:
            self.backend.queue_bind(queue=self.queue,
                                    exchange=self.exchange,
                                    routing_key=routing_key,
                                    arguments=arguments)
        self._closed = False
        return self


class Broker(object):
    def __init__(self):
        self.consumers = []

    def add_consumer(self, consumer):
        self.establish_consumer_connection(consumer)
        self.consumers.append(consumer)

    def establish_consumer_connection(self, consumer):
        # for now all consumers use the same queue connection
        # That may not be the case forever
        config = conf.config_with("rabbit_broker")
        reconnect_delay = int(config("reconnect_delay"))
        max_wait = int(config("max_wait"))

        # This just sets the connection string. It doesn't actually
        # connect to the AMQP server yet.
        connection = BrokerConnection(
                        hostname=config("host"),
                        port=config("port"),
                        userid=config("user"),
                        password=config("password"),
                        virtual_host=config("vhost"))

        auto_delete = consumer.config("auto_delete") == "True" or False
        durable = consumer.config("durable") == "True" or False

        exdurable = confbool(consumer.config("exchange_durable"))
        exauto_delete = confbool(consumer.config("exchange_auto_delete"))

        # try a few times to connect, we might have lost the connection
        retries = 0
        while True:
            try:
                carrot_consumer = NotQuiteSoStupidConsumer(
                        connection=connection,
                        warn_if_exists=True,
                        exchange=consumer.config("exchange"),
                        exchange_type=consumer.config("exchange_type"),
                        routing_key=consumer.config("routing_key"),
                        queue=consumer.queue_name,
                        auto_delete=auto_delete,
                        durable=durable,
                        exchange_durable=exdurable,
                        exchange_auto_delete=exauto_delete,
                        )
                consumer.connect(connection, carrot_consumer)
                LOG.info("Connection established for %s" % consumer.queue_name)
                break
            except amqplib.client_0_8.exceptions.AMQPConnectionException, e:
                LOG.error("AMQP protocol error connecting to for queue %s" %
                           consumer.queue_name)
                LOG.exception(e)
            except socket.error, e:
                # lost connection?
                pass
            delay = reconnect_delay * retries
            if delay > max_wait:
                delay = max_wait
            retries += 1
            LOG.error("Could not reconnect, trying again in %d" % delay)
            time.sleep(delay)

    def loop(self):
        poll_delay = float(conf.get("rabbit_broker", "poll_delay"))
        update_timer = int(conf.get("global", "update_timer"))
        max_connection_age = int(conf.get("rabbit_broker",
                                          "max_connection_age"))
        start_time = datetime.datetime.now()
        messages_sent = {}
        while True:
            try:
                for consumer in self.consumers:
                    messages = []
                    if not consumer.queue_name in messages_sent:
                        messages_sent[consumer.queue_name] = 0

                    for n in xrange(consumer.max_messages):
                        msg = consumer.consumer.fetch(enable_callbacks=False)
                        if not msg:
                            break
                        LOG.debug("Received message on queue %s" %
                            consumer.queue_name)
                        messages.append(msg)

                    num_messages = len(messages)
                    if num_messages > 0:
                        consumer.fetched_messages(messages)
                        messages_sent[consumer.queue_name] += num_messages
                    if max_connection_age > 0:
                        age = datetime.datetime.now() - consumer.connect_time
                        age_sec = age.seconds + (age.days * 86400)
                        if age_sec > max_connection_age:
                            LOG.info("Maximum AMQP connection time for "
                                     "connection to %s reached. "
                                     "Reconnecting..." % consumer.queue_name)
                            self.establish_consumer_connection(consumer)


                # Ingnoring microseconds because we're not going to let you
                # be that granular and it's not super useful anyway
                td = datetime.datetime.now() - start_time
                elapsed = td.seconds + td.days * 86400

                if elapsed > update_timer:
                    # This isn't threaded, it's just best effort
                    LOG.info("Update timer elapsed: %s seconds", str(elapsed))
                    total_messages = 0
                    for consumer in self.consumers:
                        LOG.info("\tSent %d messages from %s" %
                                          (messages_sent[consumer.queue_name],
                                           consumer.queue_name))
                        total_messages += messages_sent[consumer.queue_name]
                    LOG.info("\tSent %d total messages" % total_messages)
                    if total_messages > 0:
                        LOG.info("\tMessages per second: %f" %
                                    (float(total_messages) / elapsed))

                    start_time = datetime.datetime.now()

                    # Periodically call the consumers in case they need
                    # to do some work even when there are no events to
                    # process ...
                    for consumer in self.consumers:
                        consumer.idle(messages_sent[consumer.queue_name],
                                      consumer.queue_name)

                    messages_sent = {}
                # This should really only be used when trying to discern bugs
                # in the flood of messages and coupled with DEBUG logging.
                # Otherwise, we want Yagi sending as quickly as possible
                if poll_delay:
                    time.sleep(poll_delay)
            except socket.error, e:
                LOG.critical("Rabbit connection lost, reconnecting")
                LOG.exception(e)
                self.establish_consumer_connection(consumer)
            except amqplib.client_0_8.exceptions.AMQPException, e:
                LOG.critical("Rabbit connection lost, reconnecting")
                LOG.exception(e)
                self.establish_consumer_connection(consumer)
            except Exception, e:
                LOG.exception(e)




import errno
import httplib
import httplib2
import socket


class ResponseTooLargeError(httplib2.HttpLib2ErrorWithResponse):
    pass


class LimitingBodyHttp(httplib2.Http):

    """ This is a patched form of httplib2's Http class that is designed to
    reject reading response bodies that are too large to handle.

    By default httplib2 simply reads the whole body of the response into
    memory. This will read at most a certain size.
    """

    def __init__(self, max_body_size=1024 * 40, **kw):
        self.max_body_size = max_body_size
        self.follow_all_redirects = True
        super(LimitingBodyHttp, self).__init__(**kw)

    def _conn_request(self, conn, request_uri, method, body, headers):
        for i in range(2):
            try:
                if conn.sock is None:
                    conn.connect()
                conn.request(method, request_uri, body, headers)
            except socket.timeout:
                raise
            except socket.gaierror:
                conn.close()
                raise httplib2.ServerNotFoundError("Unable to find the server "
                                                   "at %s" % conn.host)
            except httplib2.ssl_SSLError:
                conn.close()
                raise
            except socket.error, e:
                err = 0
                if hasattr(e, 'args'):
                    err = getattr(e, 'args')[0]
                else:
                    err = e.errno
                if err == errno.ECONNREFUSED:  # Connection refused
                    raise
            except httplib.HTTPException:
                # Just because the server closed the connection doesn't
                # apparently mean that the server didn't send a response.
                if conn.sock is None:
                    if i == 0:
                        conn.close()
                        conn.connect()
                        continue
                    else:
                        conn.close()
                        raise
                if i == 0:
                    conn.close()
                    conn.connect()
                    continue
            try:
                response = conn.getresponse()
            except (socket.error, httplib.HTTPException):
                if i == 0:
                    conn.close()
                    conn.connect()
                    continue
                else:
                    raise
            else:
                content = ""
                if method == "HEAD":
                    response.close()
                else:
                    content = response.read(self.max_body_size + 1)
                    if len(content) > self.max_body_size:
                        #Too large. Drop the connection on the floor.
                        response.close()
                        conn.close()
                        raise ResponseTooLargeError(
                              "The response was larger than the maximum"
                              " size (%s) allowed" % self.max_body_size,
                              response,
                              content)
                response = httplib2.Response(response)
                if method != "HEAD":
                    content = httplib2._decompressContent(response, content)
            break
        return (response, content)



import json

import redis

import yagi.config
import yagi.persistence
from yagi.persistence import InvalidEntityUUID

with yagi.config.defaults_for('persistence') as default:
    default('host', 'localhost')
    default('port', 6379)
    default('entry_ttl', 60 * 60 * 24 * 30)
    default('password', '')


class Driver(yagi.persistence.Driver):
    def __init__(self):
        conf = yagi.config.config_with('persistence')
        host = conf('host')
        port = int(conf('port', default=6379))
        password = conf('password')
        self.ttl = int(conf('entry_ttl'))
        self.client = redis.Redis(host=host, password=password, port=port)
        super(yagi.persistence.Driver, self).__init__()

    def create(self, key, entity_uuid, value):
        if self.ttl <= 0:
            self.client.set('entry:%s:content' % entity_uuid,
                            json.dumps(value))
        else:
            self.client.setex('entry:%s:content' % entity_uuid,
                              json.dumps(value),
                              self.ttl)
        self.client.set('entry:%s:event_type' % entity_uuid, key)
        self.client.lpush('type:%s' % key, entity_uuid)
        self.client.lpush('entries', entity_uuid)

    def _clean(self, uuid):
        event_type = self.client.get('entry:%s:event_type' % uuid)
        self.client.lrem('type:%s' % event_type, uuid, 1)
        self.client.lrem('entries', uuid, 1)
        self.client.delete('entry:%s:event_type' % uuid)

    def _get(self, entity_uuid):
        content = self.client.get('entry:%s:content' % entity_uuid)
        if not content:
            self._clean(entity_uuid)
            raise InvalidEntityUUID("Invalid event uuid: %s" % entity_uuid)
        event_type = self.client.get('entry:%s:event_type' % entity_uuid)
        return {'id': entity_uuid, 'content': json.loads(content),
                'event_type': event_type}

    def get(self, key, entity_uuid):
        """key is no longer used."""
        return [self._get(entity_uuid)]

    def _get_all(self, index_name, page_size=None, page=-1):
        while True:
            bad_uuids = False
            entities = []
            length = self.client.llen(index_name)
            start, end = self._page(page, page_size, length)
            uuids = self.client.lrange(index_name, start, end)
            for uuid in uuids:
                try:
                    entities.append(self._get(uuid))
                except InvalidEntityUUID:
                    bad_uuids = True
            if not bad_uuids:
                return entities

    def get_all(self, page_size=None, page=-1):
        return self._get_all('entries', page_size, page)

    def get_all_of_type(self, key, page_size=None, page=-1):
        return self._get_all('type:%s' % key, page_size, page)

    def count(self, type_key=None):
        if not type_key:
            return self.client.llen('entries')
        else:
            return self.client.llen('type:%s' % type_key)

    def _page(self, page, pagesize, length):
        """ It may seem odd to have paging logic in the persistance drivers
        here, but this logic differs depending on the persistance store (i.e.
        a relational db would have to calculate offset and limit args, which
        are different than Redis's list indexes.) """
        if not pagesize:
            return (0, -1)
        pages = self.pages(pagesize, length)
        if page < 0:
            page = pages - (page * -1)
        if page < 0 or page >= pages:
            raise IndexError("Invalid page")
        end = (length - (page * pagesize)) - 1
        start = (end - pagesize) + 1
        start = start if start > 0 else 0
        return (start, end)



from unittest import TestCase
from yagi.handler.notification_options import NotificationOptions


class TestNotificationOptions(TestCase):
    def test_for_no_options_gives_osLicense_linux(self):
        notification_options = NotificationOptions(
            {'com.rackspace__1__options': '0'})
        options_dict = notification_options.to_cuf_options()
        self.assertEquals(options_dict, 'osLicenseType="LINUX"')

    def test_for_red_hat(self):
        notification_options = NotificationOptions(
            {'com.rackspace__1__options': '1'})
        options_dict = notification_options.to_cuf_options()
        self.assertEquals(options_dict, 'osLicenseType="RHEL"')

    def test_for_se_linux(self):
        notification_options = NotificationOptions(
            {'com.rackspace__1__options': '2'})
        options_dict = notification_options.to_cuf_options()
        self.assertEquals(options_dict, '')

    def test_for_windows(self):
        notification_options = NotificationOptions(
            {'com.rackspace__1__options': '4'})
        options_dict = notification_options.to_cuf_options()
        self.assertEquals(options_dict, 'osLicenseType="WINDOWS"')

    def test_for_windows_ms_sql(self):
        notification_options = NotificationOptions(
            {'com.rackspace__1__options': '12'})
        options_dict = notification_options.to_cuf_options()
        self.assertEquals(options_dict,
                          'osLicenseType="WINDOWS" applicationLicense="MSSQL"')

    def test_for_windows_ms_sql__web(self):
        notification_options = NotificationOptions(
            {'com.rackspace__1__options': '36'})
        options_dict = notification_options.to_cuf_options()
        self.assertEquals(options_dict,
                          'osLicenseType="WINDOWS" '
                          'applicationLicense="MSSQL_WEB"')



import logging
import yagi.commandline
from yagi.consumer import Consumer
import yagi.event_worker

LOG = logging.getLogger(__name__)

consumers = []


def create_consumer(queue_name, app=None, config=None):
    consumer = Consumer(queue_name, app, config)
    consumers.append(consumer)


def setup_consumers():
    qs = [q.strip() for q in yagi.config.get('consumers', 'queues').split(',')]
    for q in qs:
        create_consumer(q)


def start_consumers():
    if yagi.config.get('event_worker', 'daemonize') == 'True':
        import daemon
        context = daemon.DaemonContext()
        LOG.info('Starting Event Worker daemon')
        context.pidfile = yagi.config.get('event_worker', 'pidfile')
        context.open()
    yagi.event_worker.start(consumers)



import functools
import unittest

import requests
import stubout
import json

import yagi.config

from yagi.handler.stacktach_ping_handler import StackTachPing


class MockResponse(object):
    def __init__(self, status_code=200):
        self.status_code = status_code
        self.text = "Ka-Splat!!"


class MockMessage(object):
    def __init__(self, payload):
        self.payload = payload
        self.acknowledged = False

    def ack(self):
        self.acknowledged = True


class StackTachPingTests(unittest.TestCase):
    """Tests to ensure the Stachtach ping handler works"""

    def setUp(self):
        self.stubs = stubout.StubOutForTesting()
        self.config_dict = {
            'stacktach': {
                'url': 'http://127.0.0.1:9000/db/confirm/usage/exists/batch',
                'timeout': '120',
                "ping_events": "*",
                "results_from": "atompub.results",
            },
        }

        self.handler = StackTachPing()
        self.messages = [MockMessage({'event_type': 'compute.instance.create',
                    'message_id': '1',
                    'content': dict(a=3)}),
                    MockMessage({'event_type': 'compute.instance.delete',
                    'message_id': '2',
                    'content': dict(a=42)}),
                    ]
        self.mock_env = {'atompub.results':
                      {'1' : dict(code=201, error=False, message="yay",
                                  service="nova"),
                       '2' : dict(code=404, error=False, message="boo",
                                  service="nova")}}

        self.called = False
        self.data = None

        def get(*args, **kwargs):
            val = None
            for arg in args:
                if val:
                    val = val.get(arg)
                else:
                    val = self.config_dict.get(arg)
                    if not val:
                        return None or kwargs.get('default')
            return val

        def config_with(*args):
            return functools.partial(get, args)

        self.stubs.Set(yagi.config, 'config_with', config_with)
        self.stubs.Set(yagi.config, 'get', get)

    def tearDown(self):
        self.stubs.UnsetAll()

    def test_ping(self):

        def mock_put(url, data=None, **kw):
            self.called = True
            self.data = data
            return MockResponse(201)

        self.stubs.Set(requests, 'put', mock_put)
        self.stubs.Set(requests.codes, 'ok', 201)

        self.handler.handle_messages(self.messages, self.mock_env)
        self.assertTrue(self.called)
        val = json.loads(self.data)
        self.assertTrue('messages' in val)
        self.assertEqual(len(val['messages']), 2)
        self.assertEqual(val['messages']['nova']['1']['status'], 201)
        self.assertEqual(val['messages']['nova']['2']['status'], 404)

    def test_ping_fails(self):
        #make sure it doesn't blow up if stacktach is borked.

        def mock_put(url, data=None, **kw):
            self.called = True
            self.data = data
            return MockResponse(500)

        self.stubs.Set(requests, 'put', mock_put)
        self.stubs.Set(requests.codes, 'ok', 201)

        self.handler.handle_messages(self.messages, self.mock_env)
        self.assertTrue(self.called)

    def test_match_event(self):
        matched = []
        conf = self.config_dict['stacktach']
        conf['ping_events'] = "compute.instance.create"
        for payload in self.handler.iterate_payloads(self.messages,
                                                     self.mock_env):
            #should match all of them
            if self.handler.match_event(payload):
                matched.append(payload)

        self.assertEqual(len(matched), 1)

    def test_match_event_wildcard(self):
        matched = []
        for payload in self.handler.iterate_payloads(self.messages,
                                                     self.mock_env):
            #should match all of them
            if self.handler.match_event(payload):
                matched.append(payload)

        self.assertEqual(len(matched), len(self.messages))

    def test_get_results(self):
        results = self.handler.get_results(self.mock_env)
        self.assertTrue('atompub.results' in results)
        self.assertEqual(len(results),1)





"""Implements a basic statsd driver"""

# NOTE(cerberus): Implemented this way because we don't need other
# implementations at the moment, and I didn't want to introduce a
# new dependency at this time.

import socket
import logging

import yagi.config


LOG = logging.getLogger(__name__)

DRIVER = None


class StatsD(object):
    def ping(self, data):
        LOG.debug("Sending stat: %s" % data)
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        conf = yagi.config.config_with("stats")
        host = conf("host")
        port = int(conf("port"))
        try:
            sock.connect((host, port))
            sock.sendall(data)
        except socket.error:
            LOG.exception('Socket error sending statsd data.')

    def failure_message(self):
        return yagi.config.get("stats", "failure", default="yagi.failure")

    def elapsed_message(self):
        return yagi.config.get("stats", "elapsed", default="yagi.time_elapsed")

    def messages_sent(self):
        return yagi.config.get("stats", "messages_sent",
                                default="yagi.messages_sent")


class NoDriver(object):
    def ping(self, data):
        LOG.info(data)

    def failure_message(self):
        return "failure"

    def elapsed_message(self):
        return "time_elapsed"

    def messages_sent(self):
        return "messages_sent"


def time_stat(metric, value):
    """Format execution time."""
    DRIVER.ping("%s:%s|ms" % (metric, value * 1000.0))


def increment_stat(metric, value=1):
    """Format increment/decrement."""
    DRIVER.ping("%s:%s|c" % (metric, value))


def messages_sent():
    return DRIVER.messages_sent()


def elapsed_message():
    return DRIVER.elapsed_message()


def failure_message():
    return DRIVER.failure_message()


if (yagi.config.has_section("stats") and
    yagi.config.get("stats", "enabled").lower() == "true"):
    DRIVER = StatsD()
else:
    DRIVER = NoDriver()



import json
import uuid
import feedgenerator


def clean_content(cdict):
    if cdict.__class__.__name__ == 'dict':
        return dict([i for i in cdict.items() if not i[0].startswith('_')])
    return cdict


class PagedFeed(feedgenerator.Atom1Feed):

    # I hate having to do this, but there is no other way to override
    # the link generation
    def add_item_elements(self, handler, item):
        #glommed wholesale from feed generator to change *one* little thing... (mdragon)
        handler.addQuickElement(u"title", item['title'])
        if item['link'] is not None:
            handler.addQuickElement(u"link", u"", {u"href": item['link'], u"rel": u"alternate"})
        if item['pubdate'] is not None:
            handler.addQuickElement(u"updated", feedgenerator.rfc3339_date(item['pubdate']).decode('utf-8'))

        # Author information.
        if item['author_name'] is not None:
            handler.startElement(u"author", {})
            handler.addQuickElement(u"name", item['author_name'])
            if item['author_email'] is not None:
                handler.addQuickElement(u"email", item['author_email'])
            if item['author_link'] is not None:
                handler.addQuickElement(u"uri", item['author_link'])
            handler.endElement(u"author")

        # Unique ID.
        if item['unique_id'] is not None:
            unique_id = item['unique_id']
        else:
            unique_id = "urn:uuid:%s" % uuid.uuid4()
        handler.addQuickElement(u"id", unique_id)

        # Summary.
        if item['description'] is not None:
            handler.addQuickElement(u"summary", item['description'], {u"type": u"html"})

        # Enclosure.
        if item['enclosure'] is not None:
            handler.addQuickElement(u"link", '',
                {u"rel": u"enclosure",
                 u"href": item['enclosure'].url,
                 u"length": item['enclosure'].length,
                 u"type": item['enclosure'].mime_type})

        # Categories.
        for cat in item['categories']:
            handler.addQuickElement(u"category", u"", {u"term": cat})

        # Rights.
        if item['item_copyright'] is not None:
            handler.addQuickElement(u"rights", item['item_copyright'])

    # Get it to care about content elements
    def write_items(self, handler):
        for item in self.items:
            self.write_item(handler, item)

    # Get it to care about content elements
    def write_item(self, handler, item, root=False):
        handler.startElement(u"entry",
                             self.root_attributes() if root else {})
        self.add_item_elements(handler, item)
        handler.addQuickElement(u"content",
                json.dumps(clean_content(item['contents'])),
                dict(type=u'application/json'))
        handler.endElement(u"entry")

    def add_root_elements(self, handler):
        super(PagedFeed, self).add_root_elements(handler)
        if self.feed.get('next_page_url') is not None:
            handler.addQuickElement(u"link",
                                    "",
                                    {u"rel": u"next",
                                     u"href": self.feed['next_page_url']})
        if self.feed.get('previous_page_url') is not None:
            handler.addQuickElement(u"link",
                                    "",
                                    {u"rel": u"previous",
                                     u"href": self.feed['previous_page_url']})


class CufPagedFeed(feedgenerator.Atom1Feed):

    def root_attributes_for_cuf(self, title):
        if title == "Server":
            if self.feed['language'] is not None:
                return {u"xmlns:atom": self.ns, u"xml:lang": self.feed['language']}
            else:
                return {u"xmlns:atom": self.ns}
        if title == "Glance":
            return {u"xmlns:atom": self.ns,
                    u"xmlns": u"http://docs.rackspace.com/core/event",
                    u"xmlns:glance": u"http://docs.rackspace.com/usage/glance"}

    # Get it to care about content elements
    def write_item(self, handler, item, root=False, title="Server"):
        handler.startElement(u"atom:entry",
                             self.root_attributes_for_cuf(title) if root else {})
        for cat in item['categories']:
            handler.addQuickElement(u"atom:category", u"", {u"term": cat})
        handler.addQuickElement(u"atom:title", attrs={u"type": u"text"}, contents=title)
        handler.addQuickElement(u"atom:content",
                                item['contents'],
                                dict(type=u"application/xml"))
        handler.endElement(u"atom:entry")



from unittest import TestCase
import uuid
import datetime
import mox
from tests.unit import utils
from yagi.handler import notification_payload
from yagi.handler.cuf_pub_handler import Notification
from yagi.handler.notification import GlanceNotification
from yagi.handler.notification_payload import NotificationPayload
import yagi.config


class TestGlanceNotification(TestCase):

    def setUp(self):
        self.mox = mox.Mox()

    def tearDown(self):
        self.mox.UnsetStubs()

    def test_convert_to_cuf_format(self):
        self.maxDiff = None
        exists_message = {
            "event_type": "image.exists",
            "timestamp": "2013-09-02 16:09:16.247932",
            "message_id": "18b59543-2e99-4208-ba53-22726c02bd67",
            "priority": "INFO",
            "publisher_id": "ubuntu",
            "payload": {
                "images": [{
                    "status": "active",
                    "name": "image1",
                    "created_at": "2013-09-02 16:08:10",
                    "properties": {
                        "image_type": "snapshot",
                        "instance_uuid": "inst_uuid1"
                    },
                    "deleted_at": None,
                    "id": "image1",
                    "size": 12345
                },
                {
                    "status": "deleted",
                    "name": "image2",
                    "created_at": "2013-09-02 16:05:17",
                    "properties": {
                        "image_type": "snapshot",
                        "instance_uuid": "inst_uuid2"
                    },
                    "deleted_at": "2013-09-02 16:08:46",
                    "id": "image2",
                    "size": 67890
                }],
                "owner": "owner1",
                "audit_period_ending": "2013-09-02 23:59:59.999999",
                "audit_period_beginning": "2013-09-02 00:00:00"
            }
        }
        self.mox.StubOutWithMock(uuid, 'uuid4')
        uuid.uuid4().AndReturn('uuid1')
        uuid.uuid4().AndReturn('uuid2')
        self.mox.StubOutWithMock(notification_payload, 'start_time')
        self.mox.StubOutWithMock(notification_payload, 'end_time')
        notification_payload.start_time(
            '2013-09-02 16:08:10',
            '2013-09-02 00:00:00').AndReturn(datetime.datetime(2013,9,2))
        notification_payload.end_time(
            None,
            '2013-09-02 23:59:59.999999').AndReturn(datetime.datetime(
                2013, 9, 2, 23, 59, 59, 999999))
        notification_payload.start_time(
            '2013-09-02 16:05:17',
            '2013-09-02 00:00:00').AndReturn(datetime.datetime(
                2013, 9, 2, 16, 5, 17))
        notification_payload.end_time(
            '2013-09-02 16:08:46',
            '2013-09-02 23:59:59.999999').AndReturn(datetime.datetime(
                2013, 9, 2, 23, 59, 59, 999999))
        self.mox.ReplayAll()
        expected_cuf_xml = {'payload': ("""<events><event endTime="2013-09-02T23:59:59Z" """
        """startTime="2013-09-02T00:00:00Z" region="DFW" dataCenter="DFW1" """
        """type="USAGE" id="uuid1" resourceId="image1" tenantId="owner1" """
        """version="1"> <glance:product storage="12345" serverId="inst_uuid1" """
        """serviceCode="Glance" serverName="" resourceType="snapshot" """
        """version="1"/></event><event endTime="2013-09-02T23:59:59Z" """
        """startTime="2013-09-02T16:05:17Z" region="DFW" dataCenter="DFW1" """
        """type="USAGE" id="uuid2" resourceId="image2" tenantId="owner1" """
        """version="1"> <glance:product storage="67890" serverId="inst_uuid2" """
        """serviceCode="Glance" serverName="" resourceType="snapshot" """
        """version="1"/></event></events>""")}
        notification = GlanceNotification(exists_message)
        verified_message = notification.\
            convert_to_verified_message_in_cuf_format(
            {'region': 'DFW', 'data_center': 'DFW1'})

        self.assertEquals(verified_message, expected_cuf_xml)
        self.mox.VerifyAll()

    def test_start_time_should_return_launched_at_when_launched_at_is_bigger(self):
        start_time = notification_payload.start_time('2013-09-02 16:08:10',
            '2013-09-02 00:00:00')
        self.assertEqual(str(start_time), "2013-09-02 16:08:10")

    def test_end_time_should_return_deleted_at_when_deleted_at_is_smaller(self):
        end_time = notification_payload.end_time(
            '2013-09-02 16:08:46',
            '2013-09-02 23:59:59.999999')
        self.assertEqual(str(end_time), "2013-09-02 16:08:46")

    def test_end_time_should_return_audit_period_beginning_when_deleted_at_is_none(self):
        end_time = notification_payload.end_time(
            "",
            '2013-09-02 23:59:59.999999')
        self.assertEqual(str(end_time), "2013-09-02 23:59:59.999999")

class TestNovaNotification(TestCase):

    def setUp(self):
        self.mox = mox.Mox()
        self.mox.StubOutWithMock(yagi.config, 'get')
        yagi.config.get('nova', 'nova_flavor_field_name').AndReturn(
            'dummy_flavor_field_name')
        self.mox.ReplayAll()

    def tearDown(self):
        self.mox.UnsetStubs()

    def test_convert_to_cuf_format_when_launched_at_before_audit_period(self):
        exists_message = {
            'event_type': 'test',
            'message_id': 'some_uuid',
            '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
            'payload': {
                'tenant_id': '2882',
                'access_ip_v4': '5.79.20.138',
                'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                'audit_period_beginning': '2012-09-15 11:51:11',
                'audit_period_ending': '2012-09-16 11:51:11',
                'display_name': 'test',
                'bandwidth': {
                    'private': {'bw_in': 0, 'bw_out': 264902},
                    'public': {'bw_in': 1001,'bw_out': 19992}
                },
                'image_meta': {'com.rackspace__1__options': '1'},
                'instance_id': '56',
                'dummy_flavor_field_name': '10',
                'launched_at': '2012-09-14 11:51:11',
                'deleted_at': '',
                'instance_type': 'm1.nano',
                'state': 'active',
                'state_description': ''
            }
        }
        self.mox.StubOutWithMock(uuid, 'uuid4')
        uuid.uuid4().AndReturn('e53d007a-fc23-11e1-975c-cfa6b29bb814')
        self.mox.ReplayAll()
        values = utils.test_nova_xml_generator_values
        values.update({'end_time': '2012-09-16T11:51:11Z'})
        expected_cuf_xml = utils.verified_nova_message_in_cuf_format(
            values)

        notification = Notification(exists_message)
        verified_message = notification.\
            convert_to_verified_message_in_cuf_format(
            {'region':'DFW','data_center': 'DFW1'})

        self.assertEquals(expected_cuf_xml['payload'],
                          verified_message['payload'])

    def test_convert_to_cuf_format_when_launched_at_in_audit_period(self):
        audit_period_beginning = '2012-09-15 10:51:11'
        audit_period_ending = '2012-09-16 10:51:11'
        launched_at = '2012-09-15 11:51:11'
        exists_message = {
            'event_type': 'test',
            'message_id': 'some_uuid',
            '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
            'payload': {
                'tenant_id': '2882',
                'access_ip_v4': '5.79.20.138',
                'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                'audit_period_beginning': audit_period_beginning,
                'audit_period_ending': audit_period_ending,
                'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                              'public': {'bw_in': 1001,'bw_out': 19992}},
                'display_name': 'test',
                'image_meta': {'com.rackspace__1__options': '1'},
                'instance_id': '56',
                'dummy_flavor_field_name': '10',
                'launched_at': launched_at,
                'deleted_at': '',
                'instance_type': 'm1.nano',
                'state': 'active',
                'state_description': ''
            }
        }
        self.mox.StubOutWithMock(uuid, 'uuid4')
        uuid.uuid4().AndReturn('e53d007a-fc23-11e1-975c-cfa6b29bb814')
        self.mox.ReplayAll()
        values = utils.test_nova_xml_generator_values
        values.update({'end_time': '2012-09-16T10:51:11Z'})
        expected_cuf_xml = utils.verified_nova_message_in_cuf_format(
            values)
        notification = Notification(exists_message)
        verified_message = \
            notification.convert_to_verified_message_in_cuf_format(
                {'region':'DFW','data_center':'DFW1'})

        self.assertEquals(expected_cuf_xml['payload'],
                          verified_message['payload'])

    def test_convert_to_cuf_format_when_deleted_at_in_audit_period(self):
        audit_period_beginning = '2012-09-15 10:51:11'
        audit_period_ending = '2012-09-16 10:51:11'
        launched_at = '2012-09-15 11:51:11'
        deleted_at = '2012-09-15 09:51:11'
        exists_message = {
            'event_type': 'test',
            'message_id': 'some_uuid',
            '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
            'payload': {
                'tenant_id':'2882',
                'access_ip_v4': '5.79.20.138',
                'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                'audit_period_beginning': audit_period_beginning,
                'audit_period_ending': audit_period_ending,
                'display_name': 'test',
                'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                              'public': {'bw_in': 1001,'bw_out': 19992}},
                'image_meta': {'com.rackspace__1__options': '1'},
                'instance_id': '56',
                'dummy_flavor_field_name': '10',
                'launched_at': launched_at,
                'deleted_at': deleted_at,
                'instance_type': 'm1.nano',
                'state': 'active',
                'state_description': ''
            }
        }
        values = utils.test_nova_xml_generator_values
        values.update({'end_time': '2012-09-15T09:51:11Z'})
        self.mox.StubOutWithMock(uuid, 'uuid4')
        uuid.uuid4().AndReturn('e53d007a-fc23-11e1-975c-cfa6b29bb814')
        expected_cuf_xml = utils.verified_nova_message_in_cuf_format(
            values)
        notification = Notification(exists_message)
        self.mox.ReplayAll()
        verified_message = notification.\
            convert_to_verified_message_in_cuf_format(
            {'region': 'DFW', 'data_center': 'DFW1'})

        self.assertEquals(expected_cuf_xml['payload'],
                          verified_message['payload'])


class TestNotificationPayload(TestCase):

    def setUp(self):
        self.mox = mox.Mox()
        self.mox.StubOutWithMock(yagi.config, 'get')
        yagi.config.get('nova', 'nova_flavor_field_name').AndReturn(
            'dummy_flavor_field_name')
        self.mox.ReplayAll()

    def tearDown(self):
        self.mox.UnsetStubs()

    def test_start_time_should_get_max_of_launched_at_and_audit_period_beginning(self):
        payload_json ={'tenant_id':'2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 10:51:11',
                     'audit_period_ending': '2012-09-16 10:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001,'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},
                     'instance_id': '56',
                     'dummy_flavor_field_name': '10',
                     'launched_at': '2012-09-15 11:51:11',
                     'deleted_at': '2012-09-15 09:51:11',
                     'instance_type': 'm1.nano',
                     'state': 'active',
                     'state_description': ''
                    }
        payload = NotificationPayload(payload_json)
        self.assertEquals(payload.start_time,
                          datetime.datetime(2012, 9, 15, 11, 51, 11))

    def test_end_time_should_get_min_of_deleted_at_and_audit_period_ending(self):
        payload_json ={'tenant_id':'2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 10:51:11',
                     'audit_period_ending': '2012-09-16 10:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001,'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},
                     'instance_id': '56',
                     'dummy_flavor_field_name': '10',
                     'launched_at': '2012-09-15 11:51:11',
                     'deleted_at': '2012-09-15 09:51:11',
                     'instance_type': 'm1.nano',
                     'state': 'active',
                     'state_description': ''
                    }
        payload = NotificationPayload(payload_json)
        self.assertEquals(str(payload.end_time), '2012-09-15 09:51:11')

    def test_end_time_should_be_audit_period_ending_when_deleted_at_is_empty(self):
        payload_json ={'tenant_id':'2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 10:51:11',
                     'audit_period_ending': '2012-09-16 10:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001,'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},
                     'instance_id': '56',
                     'dummy_flavor_field_name': '10',
                     'launched_at': '2012-09-15 11:51:11',
                     'deleted_at': '',
                     'instance_type': 'm1.nano',
                     'state': 'active',
                     'state_description': ''
                    }
        payload = NotificationPayload(payload_json)
        self.assertEquals(payload.end_time,
                          datetime.datetime(2012, 9, 16, 10, 51, 11))

    def test_different_time_formats_should_not_raise_exception(self):
        payload_json ={'tenant_id':'2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15T10:51:11Z',
                     'audit_period_ending': '2012-09-16 10:51:11',
                     'audit_period_ending': '2012-09-16T10:51:11.799674',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001,'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},
                     'instance_id': '56',
                     'dummy_flavor_field_name': '10',
                     'launched_at': '2012-09-15 11:51:11',
                     'deleted_at': '',
                     'instance_type': 'm1.nano',
                     'state': 'active',
                     'state_description': ''
                    }
        payload = NotificationPayload(payload_json)
        self.assertEquals(payload.end_time,
                          datetime.datetime(2012, 9, 16, 10, 51, 11, 799674))




import os
import uuid
import datetime
from yagi.handler.notification_options import NotificationOptions
from yagi.handler.notification_payload import NotificationPayload, GlanceNotificationPayload

nova_cuf_template = ("""<event xmlns="http://docs.rackspace.com/core/event" """
"""xmlns:nova="http://docs.rackspace.com/event/nova" version="1" """
"""id="%(id)s" resourceId="%(instance_id)s" """
"""resourceName="%(instance_name)s" dataCenter="%(data_center)s" """
"""region="%(region)s" tenantId="%(tenant_id)s" startTime="%(start_time)s" """
"""endTime="%(end_time)s" type="USAGE"><nova:product version="1" """
"""serviceCode="CloudServersOpenStack" resourceType="SERVER" flavorId"""
"""="%(flavor_id)s" flavorName="%(flavor_name)s" status="%(status)s" """
"""%(options_string)s bandwidthIn="%(bandwidth_in)s" """
"""bandwidthOut="%(bandwidth_out)s"/></event>""")

glance_cuf_template_per_image = ("""<event endTime="%(end_time)s" """
"""startTime="%(start_time)s" region="%(region)s" """
"""dataCenter="%(data_center)s" type="USAGE" id="%(id)s" """
"""resourceId="%(resource_id)s" tenantId="%(tenant_id)s" version="1"> """
"""<glance:product storage="%(storage)s" serverId="%(server_id)s" """
"""serviceCode="Glance" serverName="%(server_name)s" """
"""resourceType="%(resource_type)s" version="1"/></event>""")

atom_hopper_time_format = "%Y-%m-%dT%H:%M:%SZ"


class Notification(object):
    def __init__(self, message):
        self.message = message

    def _create_cuf_xml(self, deployment_info, json_body):
        payload = NotificationPayload(json_body['payload'])
        notification_options = {'com.rackspace__1__options': payload.options}
        cuf_xml_values = {}
        cuf_xml_values['options_string'] = NotificationOptions(
            notification_options).to_cuf_options()
        cuf_xml_values['bandwidth_in'] = payload.bandwidth_in
        cuf_xml_values['bandwidth_out'] = payload.bandwidth_out
        cuf_xml_values['start_time'] = datetime.datetime.strftime(
            payload.start_time, atom_hopper_time_format)
        cuf_xml_values['end_time'] = datetime.datetime.strftime(
            payload.end_time, atom_hopper_time_format)
        cuf_xml_values['tenant_id'] = payload.tenant_id
        cuf_xml_values['instance_id'] = payload.instance_id
        cuf_xml_values['instance_name'] = payload.instance_name
        cuf_xml_values['id'] = uuid.uuid4()
        cuf_xml_values['flavor_id'] = payload.flavor_id
        cuf_xml_values['flavor_name'] = payload.flavor_name
        cuf_xml_values['status'] = payload.status
        cuf_xml_values['data_center'] = deployment_info['data_center']
        cuf_xml_values['region'] = deployment_info['region']
        cuf_xml = nova_cuf_template % cuf_xml_values
        return cuf_xml

    def convert_to_verified_message_in_cuf_format(self, deployment_info):
        cuf_xml = self._create_cuf_xml(deployment_info, self.message)
        return {'payload': cuf_xml}

    def get_original_message_id(self):
        return self.message.get('original_message_id', "")


class GlanceNotification(object):
    def __init__(self, message):
        self.message = message

    def _create_cuf_xml(self, deployment_info, json_body):
        payload = GlanceNotificationPayload(json_body['payload'], atom_hopper_time_format)
        images = payload.images
        cuf = "<events>"
        for image in images:
            image['data_center'] = deployment_info['data_center']
            image['region'] = deployment_info['region']
            cuf_xml = glance_cuf_template_per_image % image
            cuf += cuf_xml
        cuf += "</events>"
        return cuf

    def convert_to_verified_message_in_cuf_format(self, deployment_info):
        cuf_xml= self._create_cuf_xml(deployment_info, self.message)
        return {'payload': cuf_xml}

    def get_original_message_id(self):
        return self.message.get('original_message_id', "")



import copy
import logging
import time
import uuid

import yagi.auth
import yagi.config
import yagi.handler
from yagi.handler.http_connection import HttpConnection
from yagi.handler.http_connection import MessageDeliveryFailed
from yagi.handler.http_connection import UnauthorizedException
import yagi.serializer.atom
from yagi import stats


with yagi.config.defaults_for("atompub") as default:
    default("validate_ssl", "False")
    default("generate_entity_links", "False")
    default("retries", "-1")
    default("url", "http://127.0.0.1/nova")
    default("max_wait", "600")
    default("failures_before_reauth", "5")
    default("interval", "30")
    default("timeout", "120")


LOG = logging.getLogger(__name__)


class AtomPub(yagi.handler.BaseHandler):
    CONFIG_SECTION = "atompub"
    AUTO_ACK = True

    def note_result(self, env, payload, code=0, error=False, message=None,
                    service='nova', ah_event_id=None):
        name = self.__class__.__name__.lower() + ".results"
        results = env.get(name) if name in env else dict()
        msgid = payload["message_id"]
        result = dict(error=error)
        if message is None:
            if error:
                message = "Error, unable to send notification"
            else:
                message = "Success"
        if ah_event_id:
            result['ah_event_id'] = ah_event_id
        result['message'] = message
        result['code'] = code if not error else 0
        result['service'] = service

        results[msgid] = result
        env[name] = results

    def _should_generate_compute_instance_verified(self, is_stacktach_down,
                                                   payload,
                                                   exclude_filter_list):
        event_type = payload['event_type']
        return event_type == 'compute.instance.exists' \
            and is_stacktach_down \
            and ('compute.instance.exists.verified' not in exclude_filter_list)

    def _to_bool(self, bool_string):
        return bool_string in ['true', 'True']

    def _create_compute_instance_verified_payload(self, entity_links, payload):
        payload_copy = copy.deepcopy(payload)
        payload_copy['event_type'] = "compute.instance.exists.verified"
        entity = dict(
            content=payload_copy, id=str(uuid.uuid4()),
            event_type="compute.instance.exists.verified")
        payload_body = yagi.serializer.atom.dump_item(
            entity, entity_links=entity_links)
        return payload_body

    def _exclude_filters(self):
        exclude_filter_list = []
        exclude_filters = yagi.config.get(
            'exclude_filters', self.CONFIG_SECTION)
        if exclude_filters:
            exclude_filter_list = \
                [filter.strip() for filter in exclude_filters.split(",")]
        return exclude_filter_list

    def _create_atom_payload(self, entity_links, payload):
        entity = dict(content=payload,
                      id=payload["message_id"],
                      event_type=payload["event_type"])
        payload_body = yagi.serializer.atom.dump_item(
            entity, entity_links=entity_links)
        return payload_body

    def _send_notification(self, env, notification_payload, payload):
        retries = int(self.config_get("retries"))
        interval = int(self.config_get("interval"))
        max_wait = int(self.config_get("max_wait"))
        failures_before_reauth = int(self.config_get("failures_before_reauth"))
        endpoint = self.config_get("url")
        ah_event_id = ""
        tries = 0
        failures = 0
        code = 0
        error_msg = ''
        connection = HttpConnection(self)
        while True:
            try:
                response_details = connection.send_notification(
                    endpoint, endpoint, notification_payload)
                code = response_details.get("status")
                ah_event_id = response_details.get("ah_event_id")
                error = False
                msg = ''
                break
            except UnauthorizedException, e:
                LOG.exception(e)
                conn = None
                code = 401
                error_msg = "Unauthorized"
            except MessageDeliveryFailed, e:
                LOG.exception(e)
                code = e.code
                error_msg = e.msg
            except Exception, e:
                code = 0 #aka 'unknown failure'
                error_msg = "AtomPub General Delivery Failure to %s with: %s" % (
                    endpoint, e)
                LOG.error(error_msg)
                LOG.exception(e)

            #If we got here, something failed.
            stats.increment_stat(yagi.stats.failure_message())
            # Number of overall tries
            tries += 1
            # Number of tries between re-auth attempts
            failures += 1

            # Used primarily for testing, but it's possible we don't
            # care if we lose messages?
            if retries > 0:
                if tries >= retries:
                    msg = "Exceeded retry limit. Error %s" % error_msg
                    self.note_result(env, payload,
                                     code=code, message=msg)
                    break
            wait = min(tries * interval, max_wait)
            LOG.error("Message delivery failed, going to sleep, will "
                      "try again in %s seconds" % str(wait))
            time.sleep(wait)

            if failures >= failures_before_reauth:
                # Don't always try to reconnect, give it a few
                # tries first
                failures = 0
                connection = None
            if connection is None:
                connection = HttpConnection(self, force=True)
        return code, ah_event_id

    def handle_messages(self, messages, env):
        entity_links = self.config_get("generate_entity_links") == "True"
        is_stacktach_down = self._to_bool(self.config_get("stacktach_down"))
        exclude_filter_list = self._exclude_filters()

        for payload in self.iterate_payloads(messages, env):
            notification_payloads = []
            try:
                if self._should_generate_compute_instance_verified(
                        is_stacktach_down, payload, exclude_filter_list):
                    notification_payloads.append(
                        self._create_compute_instance_verified_payload(
                            entity_links, payload))
                notification_payloads.append(
                    self._create_atom_payload(entity_links, payload))
            except KeyError, e:
                error_msg = "Malformed Notification: %s" % payload
                LOG.error(error_msg)
                LOG.exception(e)
                self.note_result(env, payload, error=True, message=error_msg)
                continue
            for notification_payload in notification_payloads:
                code, ah_event_id = self._send_notification(env, notification_payload,
                                               payload)
                if ah_event_id:
                    self.note_result(env, payload, code=code, service='nova',
                                     ah_event_id=ah_event_id)
                else:
                    self.note_result(env, payload, code=code, service='nova')





import feedgenerator

import yagi.config
from yagi.serializer.paged_feed import PagedFeed
import yagi.utils


def _entity_link(entity_id, key):
    return unicode(''.join([_entity_url(), '%s/' % key, str(entity_id)]))


def _entity_url():
    conf = yagi.config.config_with('event_feed')
    feed_host = conf('feed_host')
    use_https = yagi.config.get_bool('event_feed', 'use_https')
    scheme = "%s://" % ('https' if use_https else 'http')
    port = conf('port') or ''
    if len(port) > 0:
        port = ':%s' % port

    if not feed_host:
        feed_host = yagi.utils.get_ip_addr()
    return unicode(''.join([scheme, feed_host, port, '/']))


def _categories():
        conf = yagi.config.config_with('event_feed')
        val = 'atom_categories'
        return [c.strip() for c in
                (conf(val).split(',') if conf(val) else [])]


def clean_content(cdict):
    return dict([i for i in cdict.items() if not i[0].startswith('_')])


def _feed_entity(feed, entity, entity_links=True):
    event_type = unicode(entity['event_type'])
    if entity_links:
        elink = _entity_link(entity['id'], entity['event_type'])
    else:
        elink = None
    feed.add_item(title=unicode(entity['event_type']),
                link=elink,
                description=event_type,
                contents=entity['content'],
                unique_id="urn:uuid:%s" % entity['id'],
                categories=[event_type] + _categories())


def dumps(entities, previous_page=None, next_page=None, entity_links=True):
    """Serializes a list of dictionaries as an ATOM feed"""

    title = unicode(yagi.config.get('event_feed', 'feed_title'))
    feed = PagedFeed(
        title=title,
        link=_entity_url(),
        feed_url=_entity_url(),
        description=title,
        language=u'en',
        previous_page_url=("%s?page=%s" % (_entity_url(), previous_page))
                          if previous_page is not None else None,
        next_page_url=("%s?page=%s" % (_entity_url(), next_page))
                          if next_page is not None else None)
    for entity in entities:
        _feed_entity(feed, entity, entity_links)
    return feed.writeString('utf-8')


def dump_item(entity, entity_links=True):
    """Serializes a single dictionary as an ATOM entry"""
    from StringIO import StringIO

    outfile = StringIO()
    handler = feedgenerator.SimplerXMLGenerator(outfile, 'utf-8')
    handler.startDocument()
    title = unicode(yagi.config.get('event_feed', 'feed_title'))
    feed = PagedFeed(
        title=title,
        link=_entity_url(),
        feed_url=_entity_url(),
        description=title,
        language=u'en',
        previous_page_url=None,
        next_page_url=None)

    _feed_entity(feed, entity, entity_links)
    feed.write_item(handler, feed.items[0], root=True)
    return outfile.getvalue()



import functools
import unittest
import uuid
import httplib2
import mox
import stubout
from yagi.handler import http_connection
import yagi
from yagi.handler.cuf_pub_handler import CufPub


class MockResponse(object):
    def __init__(self, status_code=200):
        self.status = status_code


class MockMessage(object):
    def __init__(self, payload):
        self.payload = payload
        self.acknowledged = False

    def ack(self):
        self.acknowledged = True


class CufPubTests(unittest.TestCase):
    """Tests to ensure the ATOM CUF Pub code holds together as expected"""

    def setUp(self):
        self.stubs = stubout.StubOutForTesting()
        self.mox = mox.Mox()
        config_dict = {
            'atompub': {
                'url': 'http://127.0.0.1:9000/test/%(event_type)s',
                'user': 'user',
                'key': 'key',
                'interval': 30,
                'max_wait': 600,
                'retries': 1,
                'failures_before_reauth': 5,
                'timeout': '120',
            },
            'event_feed': {
                'feed_title': 'feed_title',
                'feed_host': 'feed_host',
                'use_https': False,
                'port': 'port',
                'atom_categories': "DATACENTER=ORD1, REGION=PREPROD-ORD"

            },
            'handler_auth': {
                'method': 'no_auth'
            },
            'cufpub': {
                'url': 'http://127.0.0.1:9000/test/%(event_type)s',
                'user': 'user',
                'key': 'key',
                'interval': 30,
                'max_wait': 600,
                'retries': 1,
                'failures_before_reauth': 5,
                'timeout': '120',

            },
            'nova': {
                'nova_flavor_field_name': 'dummy_flavor_field_name'
            }
        }

        self.handler = CufPub()

        def get(*args, **kwargs):
            val = None
            for arg in args:
                if val:
                    val = val.get(arg)
                else:
                    val = config_dict.get(arg)
                    if not val:
                        return None or kwargs.get('default')
            return val

        def config_with(*args):
            return functools.partial(get, args)

        self.stubs.Set(yagi.config, 'config_with', config_with)
        self.stubs.Set(yagi.config, 'get', get)

    def tearDown(self):
        self.stubs.UnsetAll()
        self.mox.UnsetStubs()

    def test_notify_for_instance_exists_message(self):
        messages = [MockMessage(
            {
                'event_type': 'compute.instance.exists',
                'message_id': 'some_uuid',
                'original_message_id': 'original_uuid',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'display_name': 'test',
                     'bandwidth': {'private': {'bw_in': '0', 'bw_out': '264902'},
                                   'public': {'bw_in': '1001', 'bw_out': '19992'}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'dummy_flavor_field_name': '10',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': '',
                     'instance_type': 'm1.nano',
                     'state': 'active',
                     'state_description': ''}
            }
        )]

        body = ("""<?xml version="1.0" encoding="utf-8"?>\n"""
        """<atom:entry xmlns:atom="http://www.w3.org"""
        """/2005/Atom"><atom:category term="compute.instance."""
        """exists.verified.cuf"></atom:category><atom:category term"""
        """="original_message_id:original_uuid">"""
        """</atom:category><atom:title type="text">Server</atom:title>"""
        """<atom:content type="application/xml"><event xmlns="http://"""
        """docs.rackspace.com/core/event" xmlns:nova="http://docs"""
        """.rackspace.com/event/nova" version="1" """
        """id="e53d007a-fc23-11e1-975c-cfa6b29bb814" resourceId="56" """
        """resourceName="test" dataCenter="ORD1" region="PREPROD-ORD" """
        """tenantId="2882" startTime="2012-09-15T12:51:11Z" """
        """endTime="2012-09-16T11:51:11Z" type="USAGE"><nova:product """
        """version="1" serviceCode="CloudServersOpenStack" resourceType"""
        """="SERVER" flavorId="10" flavorName="m1.nano" status="ACTIVE" """
        """osLicenseType="RHEL" bandwidthIn="1001" bandwidthOut"""
        """="19992"/></event></atom:content></atom:entry>""")
        self.mox.StubOutWithMock(httplib2.Http, """request""")
        self.mox.StubOutWithMock(uuid, 'uuid4')
        content = ("""<atom:entry xmlns:atom="http://www.w3.org/2005/Atom">"""
        """<atom:id>urn:uuid:95347e4d-4737-4438-b774-6a9219d78d2a</atom:id""")
        uuid.uuid4().AndReturn('e53d007a-fc23-11e1-975c-cfa6b29bb814')
        uuid.uuid4().AndReturn('e53d007a-fc23-11e1-975c-cfa6b29bb815')
        httplib2.Http.request('http://127.0.0.1:9000/test/%(event_type)s',
                              'POST', body=body,
                              headers={'Content-Type': 'application/atom+xml'}
        ).AndReturn((MockResponse(201), content))

        self.mox.ReplayAll()

        self.handler.handle_messages(messages, dict())

        self.mox.VerifyAll()

    def test_notify_for_image_exists_message_for_one_image(self):
        messages = [MockMessage(
            {"event_type": "image.exists",
            "timestamp": "2013-09-02 16:09:16.247932",
            "message_id": "18b59543-2e99-4208-ba53-22726c02bd67",
            "original_message_id": "some_uuid",
            "priority": "INFO",
            "publisher_id": "ubuntu",
            "payload": {
                "owner": "owner1",
                "audit_period_ending": "2013-09-02 23:59:59.999999",
                "audit_period_beginning": "2013-09-02 00:00:00",
                "images": [{
                    "status": "active",
                    "name": "image1",
                    "created_at": "2013-09-02 16:08:10",
                    "properties": {
                        "image_type": "snapshot",
                        "instance_uuid": "inst_uuid1"
                    },
                    "deleted_at": None,
                    "id": "image1",
                    "size": 12345
                }]
        }}
        )]

        cuf_xml_body = ("""<?xml version="1.0" encoding="utf-8"?>\n"""
        """<atom:entry xmlns="http://docs.rackspace."""
        """com/core/event" xmlns:atom="http://www.w3.org/2005/Atom" """
        """xmlns:glance="http://docs.rackspace.com/usage/glance">"""
        """<atom:category term="image.exists.verified.cuf"></atom:category>"""
        """<atom:category term="original_message_id:some_uuid"></atom:category>"""
        """<atom:title type="text">Glance"""
        """</atom:title><atom:content type="application/xml"><events><"""
        """event endTime="2013-09-02T23:59:59Z" """
        """startTime="2013-09-02T16:08:10Z" region="ORD1" """
        """dataCenter="PREPROD-ORD" type="USAGE" """
        """id="a70508f3-7254-4abc-9e14-49de6bb4d628" resourceId="image1" """
        """tenantId="owner1" version="1"> <glance:product """
        """storage="12345" serverId="inst_uuid1" serviceCode="Glance" """
        """serverName="" resourceType="snapshot" version="1"/></event></events>"""
        """</atom:content></atom:entry>""")

        content = ("""<atom:entry xmlns:atom="http://www.w3.org/2005/Atom">"""
        """<atom:id>urn:uuid:95347e4d-4737-4438-b774-6a9219d78d2a</atom:id""")
        self.mox.StubOutWithMock(httplib2.Http, 'request')
        httplib2.Http.request('http://127.0.0.1:9000/test/%(event_type)s',
                              'POST', body=cuf_xml_body,
                              headers={'Content-Type': 'application/atom+xml'}
        ).AndReturn((MockResponse(201), content))
        self.mox.StubOutWithMock(uuid, 'uuid4')
        uuid.uuid4().AndReturn("a70508f3-7254-4abc-9e14-49de6bb4d628")
        uuid.uuid4().AndReturn("a70508f3-7254-4abc-9e14-49de6bb4d628")
        self.mox.ReplayAll()

        self.handler.handle_messages(messages, dict())

        self.mox.VerifyAll()

    def test_notify_for_image_exists_message_for_more_than_one_image(self):
        messages = [MockMessage(
            {"event_type": "image.exists",
            "timestamp": "2013-09-02 16:09:16.247932",
            "message_id": "18b59543-2e99-4208-ba53-22726c02bd67",
            "original_message_id": "some_uuid",
            "priority": "INFO",
            "publisher_id": "ubuntu",
            "payload": {
                "owner": "owner1",
                "audit_period_ending": "2013-09-02 23:59:59.999999",
                "audit_period_beginning": "2013-09-02 00:00:00",
                "images": [{
                    "status": "active",
                    "name": "image1",
                    "created_at": "2013-09-02 16:08:10",
                    "properties": {
                        "image_type": "snapshot",
                        "instance_uuid": "inst_uuid1"
                    },
                    "deleted_at": None,
                    "id": "image1",
                    "size": 12345
                },
                {
                    "status": "deleted",
                    "name": "image2",
                    "created_at": "2013-09-02 16:05:17",
                    "properties": {
                        "image_type": "snapshot",
                        "instance_uuid": "inst_uuid2"
                    },
                    "deleted_at": "2013-09-02 16:08:46",
                    "id": "image2",
                    "size": 67890
                }]
        }}
        )]

        cuf_xml_body = ("""<?xml version="1.0" encoding="utf-8"?>\n"""
        """<atom:entry xmlns="http://docs.rackspace."""
        """com/core/event" xmlns:atom="http://www.w3.org/2005/Atom" """
        """xmlns:glance="http://docs.rackspace.com/usage/glance">"""
        """<atom:category term="image.exists.verified.cuf"></atom:category>"""
        """<atom:category term="original_message_id:some_uuid"></atom:category>"""
        """<atom:title type="text">Glance</atom:title><atom:content type="application/xml">"""
        """<events><event endTime="2013-09-02T23:59:59Z" startTime="2013-09-02T"""
        """16:08:10Z" region="ORD1" dataCenter="PREPROD-ORD" type="USAGE" """
        """id="a70508f3-7254-4abc-9e14-49de6bb4d628" resourceId="image1" """
        """tenantId="owner1" version="1"> <glance:product storage="12345" """
        """serverId="inst_uuid1" serviceCode="Glance" serverName="" """
        """resourceType="snapshot" version="1"/></event><event """
        """endTime="2013-09-02T16:08:46Z" startTime="2013-09-02T16:05:17Z" """
        """region="ORD1" dataCenter="PREPROD-ORD" type="USAGE" """
        """id="a70508f3-7254-4abc-9e14-49de6bb4d628" resourceId="image2" """
        """tenantId="owner1" version="1"> <glance:product storage="67890" """
        """serverId="inst_uuid2" serviceCode="Glance" serverName="" """
        """resourceType="snapshot" version="1"/></event></events>"""
        """</atom:content></atom:entry>""")

        content = ("""<atom:entry xmlns:atom="http://www.w3.org/2005/Atom">"""
        """<atom:id>urn:uuid:95347e4d-4737-4438-b774-6a9219d78d2a</atom:id""")
        self.mox.StubOutWithMock(httplib2.Http, 'request')
        httplib2.Http.request('http://127.0.0.1:9000/test/%(event_type)s',
                              'POST', body=cuf_xml_body,
                              headers={'Content-Type': 'application/atom+xml'}
        ).AndReturn((MockResponse(201), content))
        self.mox.StubOutWithMock(uuid, 'uuid4')
        for i in range(0,3):
            uuid.uuid4().AndReturn("a70508f3-7254-4abc-9e14-49de6bb4d628")
        self.mox.ReplayAll()

        self.handler.handle_messages(messages, dict())

        self.mox.VerifyAll()

    def test_notify_fails(self):
        messages = [MockMessage(
            {
                'event_type': 'compute.instance.exists',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'dummy_flavor_field_name': '40',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': '',
                     'instance_type': 'm1.nano',
                     'state': 'active',
                     'state_description': ''}
            }
        )]
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(404), None

        self.stubs.Set(httplib2.Http, 'request', mock_request)
        self.handler.handle_messages(messages, dict())
        self.assertEqual(self.called, True)

    def test_malformed_message_should_not_raise_exception(self):
        messages = [MockMessage(
            {
                'event_type': 'compute.instance.exists',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'dummy_flavor_field_name': '40',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': ''}
            }
        )]
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(404), None

        self.stubs.Set(httplib2.Http, 'request', mock_request)
        self.handler.handle_messages(messages, dict())
        self.assertEqual(self.called, False)




import logging
from logging.handlers import WatchedFileHandler
from logging.config import fileConfig

import yagi.config

logger = logging

with yagi.config.defaults_for('logging') as default:
    default('logfile', 'yagi.log')
    default('default_level', 'INFO')
    default('logger', 'logging')
    default("config_file", "")

FORMAT = "%(name)s[%(levelname)s at %(asctime)s line: %(lineno)d] "\
         "%(message)s"


class YagiLogger(logging.Logger):
    def __init__(self, name, level=None):
        formatter = logging.Formatter(FORMAT)
        logging.Logger.__init__(self, name,
            logging.getLevelName(yagi.config.get('logging', 'default_level')))
        handlers = []
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(formatter)
        handlers.append(stream_handler)
        logfile = yagi.config.get('logging', 'logfile')
        if logfile:
            file_handler = WatchedFileHandler(filename=logfile)
            file_handler.setFormatter(formatter)
            handlers.append(file_handler)
        for handler in handlers:
            logging.Logger.addHandler(self, handler)


def setup_logging():
    config_file = yagi.config.get('logging', 'config_file')
    if config_file:
        #if a logging configfile exists, it overrides everything else.
        fileConfig(config_file)
    else:
        logging.root = YagiLogger("YagiLogger")
        logging.setLoggerClass(YagiLogger)



import mock
import unittest

from yagi import config

from yagi.handler import shoebox_handler


class Fake(object):
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs


class TestShoeboxHandler(unittest.TestCase):
    def test_init(self):
        config.config = mock.MagicMock()
        config.config.items.return_value = ()
        sh = shoebox_handler.ShoeboxHandler()

        self.assertTrue(sh.roll_checker is None)
        self.assertEqual(sh.working_directory, '.')
        self.assertEqual(sh.destination_folder, '.')
        self.assertTrue(sh.roll_manager is not None)

    def test_init_full(self):
        config.config = mock.MagicMock()
        config.config.items.return_value = {
            'roll_checker': 'tests.unit.test_shoebox_handler:Fake',
            'working_directory': 'foo',
            'destination_folder': 'blah',
            'filename_template': 'mytemplate',
            'callback': 'tests.unit.test_shoebox_handler:Fake',
            'roll_manager': 'tests.unit.test_shoebox_handler:Fake',
        }.items()
        with mock.patch.object(shoebox_handler.os.path, "isdir") as isdir:
            isdir.return_value = True
            sh = shoebox_handler.ShoeboxHandler()
            self.assertTrue(sh.roll_checker is not None)
            self.assertEqual(sh.working_directory, 'foo')
            self.assertEqual(sh.destination_folder, 'blah')
            self.assertTrue(sh.roll_manager is not None)

    def test_json_roll_manager(self):
        config.config = mock.MagicMock()
        config.config.items.return_value = {
                'working_directory': 'foo',
                'roll_manager':
                    'shoebox.roll_manager:WritingJSONRollManager'}.items()
        with mock.patch.object(shoebox_handler.os.path, "isdir") as isdir:
            isdir.return_value = True
            sh = shoebox_handler.ShoeboxHandler()
            self.assertTrue(sh.roll_manager is not None)
            self.assertEquals(sh.roll_manager.directory, 'foo')




import fcntl
import socket
import struct
import sys


def import_class(klass_str):
    module, sep, klass = klass_str.rpartition('.')
    try:
        __import__(module)
        return getattr(sys.modules[module], klass)
    except (ImportError, ValueError, AttributeError):
        raise Exception("Could not load class %s" % klass_str)


def import_module(mod_str):
    try:
        __import__(mod_str)
        return sys.modules[mod_str]
    except (ImportError, ValueError, AttributeError):
        raise Exception("Could not load module %s" % mod_str)


def get_ip_addr():
    """Try to get a default. Override if this isn't your default NIC"""

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    def addr_by_iface(iface):
        return socket.inet_ntoa(fcntl.ioctl(
            s.fileno(),
            0x8915,  # SIOCGIFADDR
            struct.pack('256s', 'eth0'))[20:24])
    ifaces = ['eth0', 'en0']
    for iface in ifaces:
        try:
            return addr_by_iface(iface)
        except Exception:
            pass
    return '127.0.0.1'



import os
import time
import logging
import uuid
from yagi import stats
import yagi
from yagi.config import config
from yagi.handler.http_connection import HttpConnection, InvalidContentException
from yagi.handler.http_connection import MessageDeliveryFailed
from yagi.handler.http_connection import UnauthorizedException
from yagi.handler.notification import Notification, GlanceNotification
import yagi.serializer.cuf

with yagi.config.defaults_for("cufpub") as default:
    default("validate_ssl", "False")
    default("generate_entity_links", "False")
    default("retries", "-1")
    default("url", "http://127.0.0.1/nova")
    default("max_wait", "600")
    default("failures_before_reauth", "5")
    default("interval", "30")
    default("timeout", "120")

LOG = logging.getLogger(__name__)

class CufPub(yagi.handler.BaseHandler):
    CONFIG_SECTION = "cufpub"
    AUTO_ACK = True

    def unescape_strings(self, payload_body):
        return str.replace(str.replace(payload_body, "&lt;", "<"), "&gt;", ">")

    def nova_cuf(self, deployment_info, payload):
        notification = Notification(payload)
        cuf = notification.convert_to_verified_message_in_cuf_format(
            {'region': deployment_info['REGION'],
             'data_center': deployment_info['DATACENTER']})
        entity = dict(content=cuf,
                      id=str(uuid.uuid4()),
                      event_type='compute.instance.exists.verified.cuf',
                      original_message_id=notification.get_original_message_id())
        payload_body = yagi.serializer.cuf.dump_item(entity)
        return self.unescape_strings(payload_body)


    def glance_cuf(self, deployment_info, payload):
        glance_notification = GlanceNotification(payload)
        cuf = glance_notification.convert_to_verified_message_in_cuf_format(
            {'region': deployment_info['DATACENTER'],
             'data_center': deployment_info['REGION']})

        entity = dict(content=cuf,
                      id=str(uuid.uuid4()),
                      event_type='image.exists.verified.cuf',
                      original_message_id=glance_notification.get_original_message_id())
        payload_body = yagi.serializer.cuf.dump_item(entity, service_title="Glance")
        return self.unescape_strings(payload_body)

    def get_deployment_info(self, deployment_info_string):
        data_center = deployment_info_string.split(',')[0].split('=')[1]
        region = deployment_info_string.split(',')[1].split('=')[1]
        deployment_info = {'DATACENTER': data_center, 'REGION': region}
        return deployment_info

    def handle_messages(self, messages,env):
        retries = int(self.config_get("retries"))
        interval = int(self.config_get("interval"))
        max_wait = int(self.config_get("max_wait"))
        failures_before_reauth = int(self.config_get("failures_before_reauth"))
        connection = HttpConnection(self)
        results = env.setdefault('cufpub.results', dict())

        for payload in self.iterate_payloads(messages, env):
            msgid = payload["message_id"]
            try:
                deployment_info_string = yagi.config.get('event_feed',
                                                  'atom_categories')
                deployment_info = self.get_deployment_info(
                    deployment_info_string)
                payload_body = ""
                if "instance.exists" in payload['event_type']:
                    service = 'nova'
                    payload_body = self.nova_cuf(deployment_info, payload)
                elif "image.exists" in payload['event_type']:
                    service = 'glance'
                    payload_body = self.glance_cuf(deployment_info, payload)
            except KeyError, e:
                error_msg = "Malformed Notification: %s" % payload
                LOG.error(error_msg)
                LOG.exception(e)
                results[msgid] = dict(error=True, code=0, message=error_msg,
                                      service=service)
                continue

            endpoint = self.config_get("url")
            tries = 0
            failures = 0
            code = 0
            ah_event_id = None
            while True:
                try:
                    response_details = connection.send_notification(endpoint, endpoint,
                                                         payload_body)
                    code = response_details.get("status")
                    ah_event_id = response_details.get("ah_event_id")
                    break
                except InvalidContentException, e:
                    LOG.exception(e)
                    LOG.error(payload_body)
                    results[msgid] = dict(error=False, code=code, message=e.msg)
                    break
                except UnauthorizedException, e:
                    LOG.exception(e)
                    connection = None
                    code = 401
                    error_msg = "Unauthorized"
                except MessageDeliveryFailed, e:
                    LOG.exception(e)
                    code = e.code
                    error_msg = e.msg
                except Exception, e:
                    code = 0 #aka 'unknown failure'
                    error_msg = "CufPub General Delivery Failure to %s with: %s" % (endpoint, e)
                    LOG.error(error_msg)
                    LOG.exception(e)

                #If we got here, something failed.
                stats.increment_stat(yagi.stats.failure_message())
                # Number of overall tries
                tries += 1
                # Number of tries between re-auth attempts
                failures += 1

                # Used primarily for testing, but it's possible we don't
                # care if we lose messages?
                if retries > 0:
                    if tries >= retries:
                        msg = "Exceeded retry limit. Error %s" % error_msg
                        results[msgid] = dict(error=False, code=code, message=msg)
                        break
                wait = min(tries * interval, max_wait)
                LOG.error("Message delivery failed, going to sleep, will "
                         "try again in %s seconds" % str(wait))
                time.sleep(wait)

                if failures >= failures_before_reauth:
                    # Don't always try to reconnect, give it a few
                    # tries first
                    failures = 0
                    connection = None
                if connection is None:
                    connection = HttpConnection(self,force=True)

            results[msgid] = dict(error=False, code=code, message="Success",
                                  service=service, ah_event_id=ah_event_id)



import unittest
import httplib2

import stubout

from yagi import auth
from yagi import config


class MockResponse(object):
    def __init__(self, status_code=200):
        self.status = status_code


class TestAuth(unittest.TestCase):

    def setUp(self):
        self.stubs = stubout.StubOutForTesting()
        config_dict = {
            'handler_auth': {
                'method': 'rax_auth_v2',
                'user': 'a_user',
                'key': 'seekret_key',
                'validate_ssl': 'False',
                'auth_server': 'http://127.0.0.1:12345/test',
            },
        }

        self.auth_json = '{"access": {"token": {"id": "%s"}}}'

        def get(*args, **kwargs):
            val = None
            for arg in args:
                if val:
                    val = val.get(arg)
                else:
                    val = config_dict.get(arg)
                    if not val:
                        return None or kwargs.get('default')
            return val

        def config_with(*args):
            return functools.partial(get, args)

        self.stubs.Set(config, 'config_with', config_with)
        self.stubs.Set(config, 'get', get)

    def tearDown(self):
        self.stubs.UnsetAll()

    def test_rax_auth_caches_token(self):
        self.calls = 0

        def mock_request(*args, **kwargs):
            self.calls += 1
            body = self.auth_json % ("token_%s" % self.calls)
            return MockResponse(200), body
        self.stubs.Set(httplib2.Http, 'request', mock_request)
        headers = dict()
        auth.rax_auth_v2("bogus_unused_conn", headers, force=False)
        self.assertEquals(self.calls, 1)
        self.assertEquals(headers['X-Auth-Token'], 'token_1')
        #round 2
        headers = dict()
        auth.rax_auth_v2("bogus_unused_conn", headers, force=False)
        self.assertEquals(self.calls, 1)
        self.assertEquals(headers['X-Auth-Token'], 'token_1')
        #round 3
        headers = dict()
        auth.rax_auth_v2("bogus_unused_conn", headers, force=True)
        self.assertEquals(self.calls, 2)
        self.assertEquals(headers['X-Auth-Token'], 'token_2')




import functools
import logging
import os

from contextlib import contextmanager
from ConfigParser import SafeConfigParser, NoOptionError, NoSectionError


CONFIG_FILE = 'yagi.conf'
CONFIG_PATHS = ["/etc/yagi/", "./", "/etc/"]

config = None
config_path = None
config_defaults = {'global': {'verbose': 'True',
                               'debug': 'False'},
                    # Defining defaults for this here, as it's usage is spread
                    # out all over the code. -mdragon.
                    'event_feed': {'pidfile': 'yagi_feed.pid',
                                   'daemonize': 'False',
                                   'port': 8080,
                                   'serializer_driver': 'yagi.serializer.atom',
                                   'feed_title': 'Notifications'}}

filters = {}


class DefaultConfigParser(SafeConfigParser):

    def get(self, section, option, raw=False, vars=None):
        try:
            return SafeConfigParser.get(self, section, option, raw, vars)
        except NoSectionError:
            try:
                section_defaults = config_defaults[section]
            except KeyError:
                raise NoSectionError(section)
            try:
                return section_defaults[option]
            except KeyError:
                raise NoOptionError(option, section)
        except NoOptionError:
            try:
                section_defaults = config_defaults[section]
                return section_defaults[option]
            except KeyError:
                raise NoOptionError(option, section)


def setup(**kwargs):
    if kwargs['config_path']:
        parse_conf(kwargs['config_path'])


def parse_conf(path=None):
    global config, config_path
    config_path = path
    config = DefaultConfigParser()
    if not config_path:
        for path in CONFIG_PATHS:
            path = path + CONFIG_FILE
            if os.path.exists(path):
                config_path = path
                break
    else:
        if not os.path.exists(path):
            raise Exception("No configuration '%s' found" % path)
    if config_path:
        config.read(config_path)
    return config


def defaults(section, option, value):
    section_defaults = config_defaults.get(section) or dict()
    section_defaults[option] = str(value)
    config_defaults[section] = section_defaults


@contextmanager
def defaults_for(section):
    yield functools.partial(defaults, section)


def lazy_load_config(fun):
    def decorate(*args, **kwargs):
        LOG = logging.getLogger(__name__)
        global config
        if not config:
            config = parse_conf()
        try:
            return fun(*args, **kwargs)
        except (NoSectionError, NoOptionError) as e:
            LOG.debug(e)
            return kwargs.get('default', None)
    return decorate


@lazy_load_config
def get(*args, **kwargs):
    return config.get(*args)


@lazy_load_config
def has_section(*args):
    return config.has_section(*args)


@lazy_load_config
def get_bool(*args, **kwargs):
    return config.getboolean(*args)


def config_with(*args):
    return functools.partial(get, *args)



import argparse


args = None


def parse_args(desc):
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('--config', '-c', help='The path to the config file')
    args = parser.parse_args()
    return args



import eventlet
from eventlet import wsgi
import logging
import webob
import webob.dec

import yagi.config
import yagi.persistence
import yagi.serializer

LOG = logging.getLogger(__name__)

with yagi.config.defaults_for('event_feed') as default:
    default('pagesize', '1000')
    default('port', '8080')


class EventFeed(object):
    def __init__(self):
        self.db_driver = yagi.persistence.persistence_driver()
        self.feed_serializer = yagi.serializer.feed_serializer()
        self.pagesize = int(yagi.config.get('event_feed', 'pagesize'))

    @webob.dec.wsgify()
    def route_request(self, req):
        path = req.environ['PATH_INFO'][1:].split('/')
        resource = path[0]
        path = filter(lambda x: len(x) > 0, path)
        if len(path) > 2:
            raise Exception("Invalid resource")
        if len(path) == 2:
            index = path[1]
            return self.get_one(req, resource, index)
        elif len(resource) > 0:
            return self.get_all_of_resource(req, resource)
        return self.get_all(req)

    def _get_page(self, req, key=None):
        maxpage = self.db_driver.pages(self.pagesize,
                                     self.db_driver.count(key)) - 1
        if 'page' in req.str_params:
            return (int(req.str_params['page']), maxpage)
        else:
            return (maxpage, maxpage)

    def get_one(self, req, resource, uuid):
        LOG.debug('get_one %s %s' % (resource, uuid))
        elements = self.db_driver.get(resource, uuid)
        return self.respond(req, elements)

    def get_all_of_resource(self, req, resource):
        LOG.debug('get_all_of_resource %s' % resource)
        page, maxpage = self._get_page(req, resource)
        elements = self.db_driver.get_all_of_type(resource,
                                                  self.pagesize,
                                                  page)
        #print elements
        return self.respond(req, elements, page, maxpage)

    def get_all(self, req):
        LOG.debug('get_all')
        page, maxpage = self._get_page(req)
        elements = self.db_driver.get_all(self.pagesize, page)
        return self.respond(req, elements, page, maxpage)

    def respond(self, req, elements, page=0, maxpage=0):
        response = webob.Response()
        response.content_type = 'application/atom+xml'
        previous_page = next_page = None
        if page > 0:
            previous_page = page - 1
        if page < maxpage:
            next_page = page + 1
        response.body = self.feed_serializer.dumps(elements,
                                                   previous_page=previous_page,
                                                   next_page=next_page)
        return response

    def listen(self, port):
        wsgi.server(eventlet.listen(('', port)), self.route_request)


def start():
    port = int(yagi.config.get('event_feed', 'port'))
    LOG.debug('Starting feed on port %d' % port)
    event_feed = EventFeed()
    event_feed.listen(port)



import logging
import yagi.config
import yagi.utils

LOG = logging.getLogger(__name__)

with yagi.config.defaults_for('event_worker') as default:
    default('pidfile', 'yagi_event_worker.pid')
    default('daemonize', 'False')
    default('event_driver', 'yagi.broker.rabbit.Broker')


def start(consumers):
    broker = yagi.utils.import_class(yagi.config.get('event_worker',
                                                     'event_driver'))()
    for consumer in consumers:
        broker.add_consumer(consumer)
    broker.loop()



test_nova_xml_generator_values = {
    'tenant_id': 2882,
    'id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
    'resource_id': '56',
    'resource_name': 'test',
    'data_center': 'DFW1',
    'region': 'DFW',
    'start_time':'2012-09-15T11:51:11Z',
    'end_time': '2012-09-16T11:51:11Z',
    'resource_type': 'SERVER',
    'options_string': 'osLicenseType="RHEL"',
    'bandwidth_in': 1001,
    'bandwidth_out': 19992,
    'flavor_id': 10,
    'flavor_name': "m1.nano",
    'status': "ACTIVE",
}

test_glance_xml_generator_values = {
    'tenant_id': 2882,
    'id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
    'resource_id': '56',
    'data_center': 'DFW1',
    'region': 'DFW',
    'start_time':'2012-09-15T11:51:11Z',
    'end_time': '2012-09-16T11:51:11Z',
    'resource_type': 'SNAPSHOT',
    'storage': 12345,
    'server_id': 123,
    'server_name': 'X'
}


def verified_nova_message_in_cuf_format(values):
    test_nova_xml_generator_values.update(values)
    cuf_xml = ("""<event xmlns="http://docs.rackspace.com/core/event" """
    """xmlns:nova="http://docs.rackspace.com/event/nova" """
    """version="1" id="%(id)s" resourceId="%(resource_id)s" resourceName="%(resource_name)s" dataCenter="%(data_center)s" region="%(region)s" tenantId="%(tenant_id)s" """
    """startTime="%(start_time)s" endTime="%(end_time)s" type="USAGE">"""
    """<nova:product version="1" serviceCode="CloudServersOpenStack" """
    """resourceType="%(resource_type)s" flavorId="%(flavor_id)s" flavorName="%(flavor_name)s" status="%(status)s" """
    """%(options_string)s """
    """bandwidthIn="%(bandwidth_in)s" bandwidthOut="%(bandwidth_out)s"/>"""
    """</event>""") % test_nova_xml_generator_values
    return {'payload': cuf_xml}


def verified_glance_message_in_cuf_format(values):
    test_glance_xml_generator_values.update(values)
    cuf_xml = (
        """<event xmlns="http://docs.rackspace.com/core/event" """
        """xmlns:glance="http://docs.rackspace.com/event/glance" """
        """endTime="%(end_time)s" startTime="%(start_time)s" """
        """region="%(region)s" dataCenter="%(data_center)s" type="USAGE" """
        """id="%(id)s" resourceId="%(resource_id)s" """
        """tenantId="%(tenant_id)s" version="1">"""
        """<glance:product storage="1200" serverId="12309477" """
        """serviceCode="Glance" serverName="myServer" """
        """resourceType="BACKUP" version="1"/>"""
        """</event>""") % test_glance_xml_generator_values
    return {'payload': cuf_xml}





import json
import logging
import requests
import time

import yagi.config
import yagi.handler

with yagi.config.defaults_for("stacktach") as default:
    default("timeout", "120")
    default("url", "http://127.0.0.1/db/confirm/usage/exists/batch")
    default("ping_events", "compute.instance.exists.verified.old")
    default("results_from", "atompub.results")

LOG = logging.getLogger(__name__)


class StackTachPing(yagi.handler.BaseHandler):
    CONFIG_SECTION = "stacktach"

    @property
    def matching_events(self):
        events = self.config_get('ping_events')
        return [e.strip() for e in events.split(',')]

    @property
    def results_from(self):
        result_names = self.config_get('results_from')
        return [e.strip() for e in result_names.split(',')]

    def match_event(self, payload):
        event_type = payload.get('event_type')
        for e in self.matching_events:
            if event_type == e:
                return True
            if e == '*':
                return True
        return False

    def get_results(self, env):
        names = self.results_from
        results = dict(((r, env.get(r)) for r in names if r in env))
        for n in names:
            if n not in env:
                LOG.error("Stacktack ping handler cannot"
                          " find results from %s!" % n)
        return results

    def handle_messages(self, messages, env):
        results = self.get_results(env)
        if not results:
            LOG.error("Stacktack ping handler cannot find any results!")
            return
        pings = dict((n, {'nova': {}, 'glance': {}})
                     for n in results)
        for payload in self.iterate_payloads(messages, env):
            if self.match_event(payload):
                ping_msgid = msgid = payload["message_id"]
                if 'original_message_id' in payload:
                    ping_msgid = payload['original_message_id']
                for result in results:
                    st = results[result].get(msgid)
                    if st is not None:
                        code = st['code']
                        service = st['service']
                        event_id = st.get('ah_event_id')
                        pings[result][service][ping_msgid] = {'status': code}
                        if event_id:
                            pings[result][service][ping_msgid]['event_id'] = event_id
        url = self.config_get("url")
        for ping in pings.values():
            if ping:
                self._post_to_st(url, ping)

    def _check_return(self, status):
        return status == requests.codes.ok

    def _post_to_st(self, url, ping):
        timeout = int(self.config_get("timeout"))
        data = json.dumps(dict(messages=ping, version=2))
        try:
            res = requests.put(url, data=data,
                               timeout=timeout,
                               allow_redirects=True)
        except requests.exceptions.Timeout:
            LOG.error("Timeout pinging stacktach")
        if not self._check_return(res.status_code):
            LOG.error("Error posting to StackTach: %s" % res.status_code)
            LOG.error("Error message from StackTach: %s" % res.text)
        return res.status_code




class NotificationOptions(object):

    def __init__(self, options):
        self.options_bit_field = options['com.rackspace__1__options']

    def to_cuf_options(self):
        options_bit_to_dict_map = {'0':  ['LINUX'],
                                   '1':  ['RHEL'],  #RedHat
                                   '2':  [],  # SELinux not used
                                   '4':  ['WINDOWS'],
                                   '12': ['WINDOWS', 'MSSQL'],
                                   '36': ['WINDOWS', 'MSSQL_WEB'],
                                   '64': ['VYATTA']}
        options = options_bit_to_dict_map[self.options_bit_field]

        final_string = ""
        if len(options) >= 1:
            attribute_value = 'osLicenseType="%s"' % options[0]

            final_string += attribute_value
        if len(options) == 2:
            attribute_value = ' applicationLicense="%s"' % options[1]
            final_string += attribute_value
        return final_string


import BeautifulSoup
import logging

from yagi import http_util
import time

import yagi.auth
import yagi.config
import yagi.handler
import yagi.serializer.atom

LOG = logging.getLogger(__name__)

class MessageDeliveryFailed(Exception):
    def __init__(self, msg, code, *args):
        self.code = code
        self.msg = msg
        args = (msg, code) + args
        super(MessageDeliveryFailed, self).__init__(*args)


class UnauthorizedException(Exception):
    pass


class InvalidContentException(Exception):
    def __init__(self, msg, code, *args):
        self.code = code
        self.msg = msg
        args = (msg, code) + args
        super(InvalidContentException, self).__init__(*args)


class HttpConnection():
    def __init__(self, handler,force=False):
        ssl_check = not (handler.config_get("validate_ssl") == "True")
        timeout = int(handler.config_get('timeout', default=120))
        self.conn = http_util.LimitingBodyHttp(timeout=timeout,
                        disable_ssl_certificate_validation=ssl_check)
        auth_method = yagi.auth.get_auth_method()
        self.headers = {}
        self.handler = handler.CONFIG_SECTION
        if auth_method:
            try:
                auth_method(self.conn, self.headers, force=force)
            except Exception, e:
                # Auth could be jacked for some reason, slow down on failing.
                # Alternatively, if we have bad credentials, don't fill
                # up the logs crying about it.
                LOG.exception(e)
                interval = int(handler.config_get("interval"))
                time.sleep(interval)
        else:
            raise Exception("Invalid auth or no auth supplied")

    def send_notification(self, endpoint, puburl, body):
        LOG.debug("Sending message to %s with body: %s" % (endpoint, body))
        self.headers["Content-Type"] = "application/atom+xml"
        try:
            resp, content = self.conn.request(endpoint, "POST",
                                              body=body,
                                              headers=self.headers)
            response_details = {"status": resp.status}
            if resp.status == 201:
                parsed_content = BeautifulSoup.BeautifulSoup(content)
                atom_id_tag = parsed_content.find('atom:id')
                if atom_id_tag:
                    ah_event_id = atom_id_tag.string.replace('urn:uuid:', '')
                    response_details.update({"ah_event_id": ah_event_id})
                elif "compute.instance.exists.verified" in parsed_content.find("title"):
                    id_tag = parsed_content.find('id')
                    if id_tag:
                        ah_event_id = id_tag.string.replace('urn:uuid:', '')
                        response_details.update({"ah_event_id": ah_event_id})
            if resp.status == 401:
                raise UnauthorizedException("Unauthorized or token expired")
            if resp.status == 409:
                #message id already exists. this is a dup, don't resend.
                return response_details
            if resp.status == 400:
                msg = ("%s resource create failed for %s Status: "
                            "%s, %s" % (self.handler, puburl, resp.status,
                                        content))
                raise InvalidContentException(msg, resp.status)
            if resp.status != 201:
                msg = ("%s resource create failed for %s Status: "
                            "%s, %s" % (self.handler, puburl, resp.status,
                                        content))
                raise MessageDeliveryFailed(msg, resp.status)
            return response_details
        except http_util.ResponseTooLargeError, e:
            if e.response.status == 201:
                # Was successfully created. Reply was just too large.
                # Note that we DON'T want to retry this if we've gotten a 201.
                LOG.error("Response too large on successful post")
                LOG.exception(e)
            else:
                msg = ("%s resource create failed for %s. "% (self.handler,
                                                              puburl),
                       "Also, response was too large.")
                raise MessageDeliveryFailed(msg, e.response.status)



import unittest
import httplib2
import mox
from yagi import http_util
from tests.unit.test_cufpub import MockMessage, MockResponse
from yagi.handler.atompub_handler import UnauthorizedException, MessageDeliveryFailed
from yagi.handler.cuf_pub_handler import CufPub
from yagi.handler.http_connection import HttpConnection, InvalidContentException
import functools
import stubout
import yagi


class HttpConnectionTests(unittest.TestCase):

    def setUp(self):
        self.stubs = stubout.StubOutForTesting()
        self.mox = mox.Mox()
        config_dict = {
            'atompub': {
                'url': 'http://127.0.0.1:9000/test/%(event_type)s',
                'user': 'user',
                'key': 'key',
                'interval': 30,
                'max_wait': 600,
                'retries': 1,
                'failures_before_reauth': 5,
                'timeout': '120',
            },
            'event_feed': {
                'feed_title': 'feed_title',
                'feed_host': 'feed_host',
                'use_https': False,
                'port': 'port'
            },
            'handler_auth': {
                'method': 'no_auth'
            },
            'cufpub': {
                'url': 'http://127.0.0.1:9000/test/%(event_type)s',
                'user': 'user',
                'key': 'key',
                'interval': 30,
                'max_wait': 600,
                'retries': 1,
                'failures_before_reauth': 5,
                'timeout': '120',
            }
        }

        self.handler = CufPub()

        def get(*args, **kwargs):
            val = None
            for arg in args:
                if val:
                    val = val.get(arg)
                else:
                    val = config_dict.get(arg)
                    if not val:
                        return None or kwargs.get('default')
            return val

        def config_with(*args):
            return functools.partial(get, args)

        self.stubs.Set(yagi.config, 'config_with', config_with)
        self.stubs.Set(yagi.config, 'get', get)

    def tearDown(self):
        self.stubs.UnsetAll()

    def test_send_notification_successfully(self):
        self.called = False
        content = ("""<atom:entry xmlns:atom="http://www.w3.org/2005/Atom">"""
        """<atom:id>urn:uuid:95347e4d-4737-4438-b774-6a9219d78d2a</atom:id>"""
        """</atom:entry>""")

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(201), content

        handler = CufPub()
        http_conn = HttpConnection(handler)
        endpoint = handler.config_get("url")
        payload_body = {"a":"b"}
        self.mox.ReplayAll()

        self.stubs.Set(httplib2.Http, 'request', mock_request)
        http_conn.send_notification(endpoint, endpoint, payload_body)
        self.mox.VerifyAll()

    def test_response_401_raises_unauthorized_exception(self):
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(401), None

        handler = CufPub()
        http_conn = HttpConnection(handler)
        endpoint = handler.config_get("url")
        payload_body = {"a":"b"}
        self.stubs.Set(httplib2.Http, 'request', mock_request)

        self.assertRaises(UnauthorizedException,http_conn.send_notification,
                          endpoint, endpoint, payload_body)

    def test_response_other_than_201_or_400_raises_exception(self):
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(405), None

        handler = CufPub()
        http_conn = HttpConnection(handler)
        endpoint = handler.config_get("url")
        payload_body = {"a":"b"}
        self.stubs.Set(httplib2.Http, 'request', mock_request)

        self.assertRaises(MessageDeliveryFailed,http_conn.send_notification,
                          endpoint, endpoint, payload_body)

    def test_response_too_large_error_and_status_not_201_raises_exception(self):
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            raise http_util.ResponseTooLargeError("desc",MockResponse(0),
                                                  "content")

        handler = CufPub()
        http_conn = HttpConnection(handler)
        endpoint = handler.config_get("url")
        payload_body = {"a":"b"}
        self.stubs.Set(httplib2.Http, 'request', mock_request)

        self.assertRaises(MessageDeliveryFailed, http_conn.send_notification,
                          endpoint, endpoint, payload_body)

    def test_response_too_large_error_and_status_is_201_does_not_raise_exception(self):
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            raise http_util.ResponseTooLargeError("desc",MockResponse(201),
                                                  "content")

        handler = CufPub()
        http_conn = HttpConnection(handler)
        endpoint = handler.config_get("url")
        payload_body = {"a":"b"}
        self.stubs.Set(httplib2.Http, 'request', mock_request)

        http_conn.send_notification(endpoint, endpoint, payload_body)

    def test_duplicate_entry_to_atomhopper_response_409(self):
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(409), None

        handler = CufPub()
        http_conn = HttpConnection(handler)
        endpoint = handler.config_get("url")
        payload_body = {"a":"b"}
        self.stubs.Set(httplib2.Http, 'request', mock_request)

        status = http_conn.send_notification(endpoint, endpoint, payload_body)
        self.assertEqual({'status': 409}, status)

    def test_response_status_400_raises_invalid_content_exception(self):
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(400), None
        handler = CufPub()
        http_conn = HttpConnection(handler)
        endpoint = handler.config_get("url")
        payload_body = {"a":"b"}
        self.stubs.Set(httplib2.Http, 'request', mock_request)

        self.assertRaises(InvalidContentException, http_conn.send_notification,
                          endpoint, endpoint, payload_body)






import logging

import yagi.handler
import yagi.persistence

LOG = logging.getLogger(__name__)

event_attributes = ['message_id', 'publisher_id', 'event_type', 'priority',
                    'payload', 'timestamp']


class RedisHandler(yagi.handler.BaseHandler):

    def handle_messages(self, messages, env):
        db = yagi.persistence.persistence_driver()
        for payload in self.iterate_payloads(messages, env):
            self._persist_event(db, payload)

    def _persist_event(self, db, message_body):
        """Stores an incoming event in the database

        Messages have the following expected attributes:

        message_id - a UUID representing the id for this notification
        publisher_id - the source worker_type.host of the message
        timestamp - the GMT timestamp the notification was sent at
        event_type - the literal type of event (ex. Instance Creation)
        priority - patterned after the enumeration of Python logging levels in
                   the set (DEBUG, WARN, INFO, ERROR, CRITICAL)
        payload - A python dictionary of attributes
        """
        for key in event_attributes:
            if not key in message_body:
                LOG.error("Invalid Message Format, missing key %s" % key)
        event_type = message_body['event_type']
        m_id = message_body['message_id']
        db.create(event_type, m_id, message_body)
        LOG.debug('New notification created')



from ConfigParser import NoSectionError, NoOptionError
import logging
import yagi.config


LOG = logging.getLogger(__name__)


class BaseHandler(object):
    CONFIG_SECTION = "DEFAULT"
    AUTO_ACK = False

    def __init__(self, app=None, queue_name=None):
        self.app = app
        self.queue_name = queue_name

    def config_get(self, key, default=None):
        return self._config_get(yagi.config.get, key, default=default)

    def config_getbool(self, key, default=None):
        return self._config_get(yagi.config.get_bool, key, default=default)

    def _config_get(self, method, key, default=None):
        val = None
        if self.queue_name is not None:
            try:
                val = method("%s:%s" % (self.CONFIG_SECTION, self.queue_name),
                             key)
            except NoSectionError:
                pass  # nothing here, try elsewhere.
        if val is None:
            val = method(self.CONFIG_SECTION, key, default=default)
        return val

    def filter_message(self, messages):
        try:
            filter_event_type_list = []
            exclude_filter_event_type_list = []
            filter_event_type = yagi.config.get(
                'filters', self.CONFIG_SECTION)
            if filter_event_type:
                filter_event_type_list = [a.strip() for a in filter_event_type.
                                          split(",")]
            exclude_filter_event_type = yagi.config.get(
                'exclude_filters', self.CONFIG_SECTION)
            if exclude_filter_event_type:
                exclude_filter_event_type_list = [a.strip() for a in
                                                  exclude_filter_event_type.
                                                  split(",")]
            if filter_event_type:
                messages = [message for message in messages if message.payload[
                            'event_type'] in filter_event_type_list]
            if exclude_filter_event_type:
                return [message for message in messages if message.payload[
                        'event_type'] not in exclude_filter_event_type_list]
        except (NoOptionError, NoSectionError):
            pass
        return messages

    def idle(self, num_messages, queue_name):
        if self.app:
            self.app.on_idle(num_messages, queue_name)

        # Separate method call so derived class doesn't
        # need to remember to call the base class.
        self.on_idle(num_messages, queue_name)

    def on_idle(self, num_messages):
        # Do nothing. It's ok to not implement this method.
        pass

    def __call__(self, messages, env=None):
        if env is None:
            env = dict()
        if self.app:
            self.app(messages, env=env)
        filtered_messages = self.filter_message(messages)
        self.handle_messages(filtered_messages, env=env)
        return env

    def filter_payload(self, payload, env):
        filters = env.get('yagi.filters')
        if filters:
            for f in filters:
                payload = f(payload)
        return payload

    def iterate_payloads(self, messages, env):
        for message in messages:
            yield self.filter_payload(message.payload, env)
            if self.AUTO_ACK and not message.acknowledged:
                message.ack()

    def handle_messages(self, messages, env):
        raise NotImplementedError()



import yagi.config
import yagi.utils

yagi.config.defaults('persistence',
                     'driver',
                     'yagi.persistence.devnull.Driver')


class InvalidEntityUUID(KeyError):
    pass


def persistence_driver():
    driver = yagi.config.get('persistence', 'driver')
    return yagi.utils.import_class(driver)()


class Driver(object):
    def create(self, key, entity_uuid, value):
        pass

    def get(self, key, entity_uuid):
        return []

    def get_all(self):
        return []

    def get_all_of_type(self, key):
        return []

    def count(self, type_key=None):
        return 1

    def pages(self, pagesize, length):
        if not pagesize:
            return 1
        pages = (length // pagesize) + 1 if length % pagesize \
                                         else length // pagesize
        return pages if pages >= 1 else 1



import functools
import unittest
import uuid

import httplib2
import mox
import stubout

import yagi.config

from yagi.handler.atompub_handler import AtomPub


class MockResponse(object):
    def __init__(self, status_code=200):
        self.status = status_code


class MockMessage(object):
    def __init__(self, payload):
        self.payload = payload
        self.acknowledged = False

    def ack(self):
        self.acknowledged = True


class AtomPubTests(unittest.TestCase):
    """Tests to ensure the ATOM Pub code holds together as expected"""
    config_dict = {
        'atompub': {
            'url': 'http://127.0.0.1:9000/test/%(event_type)s',
            'user': 'user',
            'key': 'key',
            'interval': 30,
            'max_wait': 600,
            'retries': 1,
            'failures_before_reauth': 5,
            'stacktach_down': 'False',
            'timeout': '120',
        },
        'event_feed': {
            'feed_title': 'feed_title',
            'feed_host': 'feed_host',
            'use_https': False,
            'port': 'port'
        },
        'handler_auth': {
            'method': 'no_auth'
        },
        'exclude_filters': {
          'atompub': {}
        }
    }

    def setUp(self):
        self.stubs = stubout.StubOutForTesting()
        self.handler = AtomPub()
        self.mox = mox.Mox()

        def get(*args, **kwargs):
            val = None
            for arg in args:
                if val:
                    val = val.get(arg)
                else:
                    val = AtomPubTests.config_dict.get(arg)
                    if not val:
                        return None or kwargs.get('default')
            return val

        def config_with(*args):
            return functools.partial(get, args)

        self.stubs.Set(yagi.config, 'config_with', config_with)
        self.stubs.Set(yagi.config, 'get', get)

    def tearDown(self):
        self.stubs.UnsetAll()
        self.mox.UnsetStubs()

    def test_notify(self):
        messages = [MockMessage({'event_type': 'instance_create',
                    'message_id': 1,
                    'content': dict(a=3)})]
        content = ("""<atom:entry xmlns:atom="http://www.w3.org/2005/Atom">"""
        """<atom:id>urn:uuid:95347e4d-4737-4438-b774-6a9219d78d2a</atom:id>"""
        """</atom:entry>""")
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(201), content

        self.stubs.Set(httplib2.Http, 'request', mock_request)
        self.handler.handle_messages(messages, dict())
        self.assertEqual(self.called, True)

    def test_notify_fails(self):
        messages = [MockMessage({'event_type': 'instance_create',
                    'message_id': 1,
                    'content': dict(a=3)})]
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(404), None

        self.stubs.Set(httplib2.Http, 'request', mock_request)
        self.handler.handle_messages(messages, dict())
        self.assertEqual(self.called, True)

    def test_send_exists_and_exists_verified_when_stacktach_down(self):
        message_id = 1
        exists_payload = {'event_type': 'compute.instance.exists',
                          'message_id': message_id, 'content': dict(a=3)}
        exists_verified_payload = {
            'event_type': 'compute.instance.exists.verified',
            'message_id': message_id, 'content': dict(a=3)}
        content = ("""entry xmlns:atom="http://www.w3.org/2005/Atom">"""
        """<title type="text">compute.instance.exists.verified</title>"""
        """<id>urn:uuid:95347e4d-4737-4438-b774-6a9219d78d2a<id>"""
        """<entry>""")
        messages = [MockMessage(exists_payload)]
        self.called = False

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(201), content

        AtomPubTests.config_dict['atompub']['stacktach_down'] = 'True'
        new_message_id = '16fd2706-8baf-433b-82eb-8c7fada847da'
        self.mox.StubOutWithMock(uuid, 'uuid4')
        uuid.uuid4().AndReturn(new_message_id)
        self.mox.StubOutWithMock(yagi.serializer.atom, 'dump_item')
        expected_verified_entity = {
                    'event_type': 'compute.instance.exists.verified',
                    'id': new_message_id,
                    'content': exists_verified_payload}
        expected_exists_entity = {
                    'event_type': 'compute.instance.exists',
                    'id': message_id,
                    'content': exists_payload}
        yagi.serializer.atom.dump_item(expected_verified_entity,
                                       entity_links=False)
        yagi.serializer.atom.dump_item(expected_exists_entity,
                                       entity_links=False)
        self.mox.ReplayAll()
        self.stubs.Set(httplib2.Http, 'request', mock_request)
        self.handler.handle_messages(messages, dict())
        self.mox.VerifyAll()
        self.assertEqual(self.called, True)

    def test_send_only_exists_when_stacktach_down(self):
        payload = {'event_type': 'compute.instance.exists', 'message_id': 1,
                   'content': dict(a=3)}
        messages = [MockMessage(payload)]
        self.called = False
        content = ("""entry xmlns:atom="http://www.w3.org/2005/Atom">"""
        """<title type="text">compute.instance.exists.verified</title>"""
        """<id>urn:uuid:95347e4d-4737-4438-b774-6a9219d78d2a<id>"""
        """<entry>""")

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(201), content

        AtomPubTests.config_dict['atompub']['stacktach_down'] = 'True'
        AtomPubTests.config_dict['exclude_filters']['atompub'] = \
            'compute.instance.exists.verified'
        self.mox.StubOutWithMock(yagi.serializer.atom, 'dump_item')
        expected_entity = {
                    'event_type': 'compute.instance.exists',
                    'id': 1,
                    'content': payload}
        yagi.serializer.atom.dump_item(expected_entity, entity_links=False)
        self.mox.ReplayAll()
        self.stubs.Set(httplib2.Http, 'request', mock_request)
        self.handler.handle_messages(messages, dict())
        self.mox.VerifyAll()
        self.assertEqual(self.called, True)

    def test_do_not_change_other_events_to_verified_when_stacktach_down(self):
        payload = {'event_type': 'compute.instance.random', 'message_id': 1,
                   'content': dict(a=3)}
        messages = [MockMessage(payload)]
        self.called = False
        content = ("""<atom:entry xmlns:atom="http://www.w3.org/2005/Atom">"""
        """<atom:id>urn:uuid:95347e4d-4737-4438-b774-6a9219d78d2a</atom:id>"""
        """</atom:entry>""")

        def mock_request(*args, **kwargs):
            self.called = True
            return MockResponse(201), content

        AtomPubTests.config_dict['atompub']['stacktach_down'] = 'True'
        self.mox.StubOutWithMock(yagi.serializer.atom, 'dump_item')
        expected_entity = {
                    'event_type': 'compute.instance.random',
                    'id': 1,
                    'content': payload}
        yagi.serializer.atom.dump_item(expected_entity, entity_links=False)
        self.mox.ReplayAll()
        self.stubs.Set(httplib2.Http, 'request', mock_request)
        self.handler.handle_messages(messages, dict())
        self.mox.VerifyAll()
        self.assertEqual(self.called, True)



import functools
import unittest
import mox
import stubout
import yagi
from yagi.handler import cuf_pub_handler, atompub_handler
from yagi.handler.cuf_pub_handler import CufPub


class MockResponse(object):
    def __init__(self, status_code=200):
        self.status = status_code


class MockMessage(object):
    def __init__(self, payload):
        self.payload = payload
        self.acknowledged = False

    def ack(self):
        self.acknowledged = True


class BaseHandlerTests(unittest.TestCase):
    """Tests to ensure the ATOM CUF Pub code holds together as expected"""

    config_dict = {
        'atompub': {
            'url': 'http://127.0.0.1:9000/test/%(event_type)s',
            'user': 'user',
            'key': 'key',
            'interval': 30,
            'max_wait': 600,
            'retries': 1,
            'failures_before_reauth': 5
        },
        'event_feed': {
            'feed_title': 'feed_title',
            'feed_host': 'feed_host',
            'use_https': False,
            'port': 'port',
            'atom_categories': {'DATACENTER': 'ORD1',
                                 'REGION': 'PREPROD-ORD'}

        },
        'handler_auth': {
            'method': 'no_auth'
        },
        'cufpub': {
            'url': 'http://127.0.0.1:9000/test/%(event_type)s',
            'user': 'user',
            'key': 'key',
            'interval': 30,
            'max_wait': 600,
            'retries': 1,
            'failures_before_reauth': 5

        },
        'filters' : {
            'cufpub': 'compute.instance.exists.verified, compute.instance.exists'
        },
        'exclude_filters' : {}
    }

    def setUp(self):
        self.mox = mox.Mox()
        self.stubs = stubout.StubOutForTesting()

        self.handler = CufPub()

        def get(*args, **kwargs):
            val = None
            for arg in args:
                if val:
                    val = val.get(arg)
                else:
                    val = self.config_dict.get(arg)
                    if not val:
                        return None or kwargs.get('default')
            return val

        def config_with(*args):
            return functools.partial(get, args)

        self.stubs.Set(yagi.config, 'config_with', config_with)
        self.stubs.Set(yagi.config, 'get', get)

    def tearDown(self):
        self.mox.UnsetStubs()
        self.stubs.UnsetAll()

    def test_filter_out_exists_messages_for_cuf_handler(self):
        mock_message1 = MockMessage(
            {
                'event_type': 'compute.instance.exists',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'instance_type_id': '10',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': ''}
            }
        )
        mock_message2 = MockMessage(
            {
                'event_type': 'compute.instance.exists.verified',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'instance_type_id': '10',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': ''}
            }
        )
        mock_message3 = MockMessage(
            {
                'event_type': 'test',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'instance_type_id': '10',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': ''}
            }
        )
        messages = [mock_message1,mock_message2,mock_message3]
        filtered_messages= [mock_message1,mock_message2]
        cuf_pub = cuf_pub_handler.CufPub()
        self.mox.StubOutWithMock(cuf_pub, 'handle_messages')
        cuf_pub.handle_messages(filtered_messages,env=dict())
        self.mox.ReplayAll()
        cuf_pub(messages, dict())
        self.mox.VerifyAll()

    def test_filter_should_not_be_applied_if_not_declared_in_config(self):
        mock_message1 = MockMessage(
            {
                'event_type': 'test',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'instance_type_id': '10',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': ''}
            }
        )
        messages = [mock_message1]
        filtered_messages= [mock_message1]
        atom_pub = atompub_handler.AtomPub()
        self.config_dict['exclude_filters'] = {
            "atompub": {}}
        self.mox.StubOutWithMock(atom_pub, 'handle_messages')
        atom_pub.handle_messages(filtered_messages,env=dict())
        self.mox.ReplayAll()
        atom_pub(messages, dict())
        self.mox.VerifyAll()

    def test_exclude_filter_discards_message(self):
        mock_message1 = MockMessage(
            {
                'event_type': 'compute.instance.exists',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'instance_type_id': '10',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': ''}
            }
        )
        mock_message2 = MockMessage(
            {
                'event_type': 'image.exists.verified',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'instance_type_id': '10',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': ''}
            }
        )
        mock_message3 = MockMessage(
            {
                'event_type': 'compute.instance.exists',
                'message_id': 'some_uuid',
                '_unique_id': 'e53d007a-fc23-11e1-975c-cfa6b29bb814',
                'payload':
                    {'tenant_id': '2882',
                     'access_ip_v4': '5.79.20.138',
                     'access_ip_v6': '2a00:1a48:7804:0110:a8a0:fa07:ff08:157a',
                     'audit_period_beginning': '2012-09-15 11:51:11',
                     'audit_period_ending': '2012-09-16 11:51:11',
                     'bandwidth': {'private': {'bw_in': 0, 'bw_out': 264902},
                                   'public': {'bw_in': 1001, 'bw_out': 19992}
                                  },
                     'image_meta': {'com.rackspace__1__options': '1'},

                     'instance_id': '56',
                     'instance_type_id': '10',
                     'launched_at': '2012-09-15 12:51:11',
                     'deleted_at': ''}
            }
        )
        self.config_dict['exclude_filters'] = {
            "atompub": 'image.exists.verified'}
        messages = [mock_message1, mock_message2, mock_message3]
        filtered_messages= [mock_message1, mock_message3]
        atom_pub = atompub_handler.AtomPub()
        self.mox.StubOutWithMock(atom_pub, 'handle_messages')
        atom_pub.handle_messages(filtered_messages,env=dict())
        self.mox.ReplayAll()
        atom_pub(messages, dict())
        self.mox.VerifyAll()




import json
import logging
import os

import notification_utils
from shoebox import roll_manager
import simport

import yagi.config
import yagi.handler
import yagi.utils

LOG = logging.getLogger(__name__)


class ShoeboxHandler(yagi.handler.BaseHandler):
    """The shoebox handler doesn't really fit with
       yagi persistence interface currently. We'll need
       to refactor one or the other to clean it up.
    """

    def __init__(self, app=None, queue_name=None):
        super(ShoeboxHandler, self).__init__(app, queue_name)
        # Don't use interpolation from ConfigParser ...
        self.config = dict(yagi.config.config.items('shoebox', raw=True))
        roll_checker_str = self.config.get('roll_checker')
        self.roll_checker = None
        if roll_checker_str:
            self.roll_checker = simport.load(roll_checker_str)(**self.config)
        self.working_directory = self.config.get('working_directory', '.')
        self.destination_folder = self.config.get('destination_folder', '.')
        for d in [self.working_directory, self.destination_folder]:
            if not os.path.isdir(d):
                os.makedirs(d)
        template=self.config.get('filename_template',
                                 'events_%Y_%m_%d_%X_%f.dat')
        callback_str = self.config.get('callback')
        cb = None
        if callback_str:
            cb = simport.load(callback_str)(**self.config)

        roll_manager_str = self.config.get('roll_manager',
                                    'shoebox.roll_manager:WritingRollManager')

        self.roll_manager = simport.load(roll_manager_str)(template,
                        self.roll_checker, directory=self.working_directory,
                        archive_callback=cb)

    def handle_messages(self, messages, env):
        # TODO(sandy): iterate_payloads filters messages first ... not
        # sure if we want that for raw archiving.
        for payload in self.iterate_payloads(messages, env):
            metadata = {}
            json_event = json.dumps(payload,
                                    cls=notification_utils.DateTimeEncoder)
            LOG.debug("shoebox writing payload: %s" % str(payload))
            self.roll_manager.write(metadata, json_event)



import datetime
import dateutil.parser
import json


class FilterMessage(object):
    def __init__(self, map_file, logger):
        self.log = logger
        self.log.info("Initializing Filter: %s with Map: %s" %
                (self.__class__.__name__, map_file))
        with open(map_file, 'r') as f:
            self.transform_dict = json.load(f)

    def transform(self, mapping, message):
        for k, v in mapping.iteritems():
            if isinstance(message, dict) and message.get(k):
                if isinstance(message[k], dict):
                    self.transform(v, message[k])
                else:
                    message[k] = v
        return message

    def __call__(self, message):
        return self.transform(self.transform_dict, message)


class FilterMessageMatch(FilterMessage):
    def transform(self, mapping, message):
        for k, v in mapping.iteritems():
            if isinstance(message, dict) and message.get(k):
                if isinstance(v, dict):
                    message[k] = self.transform(v, message[k])
                elif isinstance(v, list):
                    for match_dict in v:
                        if match_dict.get(unicode(message[k])):
                            message[k] = match_dict[unicode(message[k])]
                            break
            else:
                if unicode(k) == message:
                    return v
        return message


class FilterMessageTimestamp(FilterMessage):
    def transform(self, mapping, message):
        for k, v in mapping.iteritems():
            if isinstance(message, dict) and message.get(k):
                if isinstance(message[k], dict):
                    self.transform(v, message[k])
                else:
                    try:
                        audit = dateutil.parser.parse(message[k])
                        delta = datetime.timedelta(hours=int(v))
                        offset = audit + delta

                        message[k] = offset.strftime("%Y-%m-%d %H:%M:%S.%f")
                    except Exception, e:
                        # log the exception, but there still could be other
                        # keys to convert
                        self.log.error("Bad timestamp in message: %s" %
                                                message[k])
                        self.log.exception(e)
        return message


class FilterMessageTimestampForUsageRetards(FilterMessage):
    def transform(self, mapping, message):
        for k, v in mapping.iteritems():
            if isinstance(message, dict) and message.get(k):
                if isinstance(message[k], dict):
                    self.transform(v, message[k])
                else:
                    try:
                        audit = dateutil.parser.parse(message[k])
                        delta = datetime.timedelta(hours=int(v))
                        offset = audit + delta

                        message[k] = offset.strftime("%Y-%m-%dT%H:%M:%S.%f")
                    except Exception, e:
                        # log the exception, but there still could be other
                        # keys to convert
                        self.log.error("Bad timestamp in message: %s" %
                                                message[k])
                        self.log.exception(e)
        return message


def get_filter(filter_name, map_file, logger):
    if not filter_name in globals():
        logger.exception("No filter named %s" % filter_name)
        return None
    return globals()[filter_name](map_file, logger)



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='yagi',
    version='0.13',
    author='Matthew Dietz, Monsyne Dragon',
    author_email='matthew.dietz@gmail.com, mdragon@rackspace.com',
    description=("A PubSubHubBub Publisher and ATOM feed generator that "
                   "can consume events from multiple input sources and "
                   "publish them to standard PSHB hubs"),
    license='Apache License (2.0)',
    keywords='PubSubHubBub notifications events ATOM RSS',
    packages=find_packages(exclude=['tests']),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.6'
    ],
    url='https://github.com/Cerberus98/yagi',
    scripts=['bin/yagi-feed', 'bin/yagi-event'],
    long_description=read('README.md'),
    install_requires=[
        "anyjson",
        "redis",
        "argparse",
        "carrot",
        "eventlet",
        "python-dateutil",
        "feedgenerator",
        "httplib2",
        "PubSubHubbub_Publisher",
        "requests",
        "routes",
        "WebOb",
        "python-daemon",
        "BeautifulSoup",
    ],
    zip_safe=False
)



import datetime
import logging
import uuid
import yagi

LOG = logging.getLogger(__name__)


def start_time(launched_at, audit_period_beginning):
        start_time = max(launched_at, audit_period_beginning)
        return format_time(start_time, return_in_datetime_format=True)


def end_time(deleted_at, audit_period_ending):
        if not deleted_at:
            return format_time(audit_period_ending, return_in_datetime_format=True)
        end_time = min(deleted_at, audit_period_ending)
        return format_time(end_time, return_in_datetime_format=True)


def format_time(time,return_in_datetime_format=False):
        if 'T' in time:
            try:
                # Old way of doing it
                time = datetime.datetime.strptime(time, "%Y-%m-%dT%H:%M:%S.%f")
            except ValueError:
                try:
                    # Old way of doing it, no millis
                    time = datetime.datetime.strptime(time, "%Y-%m-%dT%H:%M:%S")
                except Exception, e:
                    LOG.exception("BAD DATE: %s", time)
        else:
            try:
                time = datetime.datetime.strptime(time, "%Y-%m-%d %H:%M:%S.%f")
            except ValueError:
                try:
                    time = datetime.datetime.strptime(time, "%Y-%m-%d %H:%M:%S")
                except ValueError:
                    try:
                        time = datetime.datetime.strptime(time, "%d %m %Y %H:%M:%S")
                    except Exception, e:
                        LOG.exception("BAD DATE: %s", time)
        if return_in_datetime_format:
            return time
        return str(time)


class NotificationPayload(object):
    def __init__(self, payload_json):
        self.deleted_at = ''
        self.image_meta = payload_json.get('image_meta', {})
        self.options = self.image_meta.get('com.rackspace__1__options', '0')
        bandwidth = payload_json.get('bandwidth', {})
        public_bandwidth = bandwidth.get('public', {})
        self.bandwidth_in = public_bandwidth.get('bw_in', 0)
        self.bandwidth_out = public_bandwidth.get('bw_out', 0)

        self.launched_at = str(format_time(payload_json['launched_at']))

        self.audit_period_beginning = str(format_time(
            payload_json['audit_period_beginning']))

        self.audit_period_ending = str(format_time(
            payload_json['audit_period_ending']))

        if payload_json['deleted_at']:
            self.deleted_at = str(format_time(
                payload_json['deleted_at']))

        self.tenant_id = payload_json.get('tenant_id', "")
        self.instance_id = payload_json.get('instance_id', "")
        self.instance_name = payload_json.get('display_name', "")
        field_name = yagi.config.get('nova', 'nova_flavor_field_name')
        self.flavor_id = payload_json[field_name]
        self.flavor_name = payload_json['instance_type']
        task_state = payload_json.get('state_description', "")
        vm_state = payload_json.get('state', "")
        self.status = self._get_status(task_state, vm_state)
        self.start_time = start_time(self.launched_at,
                                     self.audit_period_beginning)
        self.end_time = end_time(self.deleted_at, self.audit_period_ending)

    def _get_status(self, task_state, vm_state):
        _STATE_MAP = {
            "active": {
                'default': 'ACTIVE',
                "rebooting": 'REBOOT',
                "rebooting_hard": 'HARD_REBOOT',
                "updating_password": 'PASSWORD',
                "rebuilding": 'REBUILD',
                "rebuild_block_device_mapping": 'REBUILD',
                "rebuild_spawning": 'REBUILD',
                "migrating": 'MIGRATING',
                "resize_prep": 'RESIZE',
                "resize_migrating": 'RESIZE',
                "resize_migrated": 'RESIZE',
                "resize_finish": 'RESIZE',
            },
            "building": {
                'default': 'BUILD',
            },
            "stopped": {
                'default': 'SHUTOFF',
            },
            "resized": {
                'default': 'VERIFY_RESIZE',
                "resize_reverting": 'REVERT_RESIZE',
            },
            "paused": {
                'default': 'PAUSED',
            },
            "suspended": {
                'default': 'SUSPENDED',
            },
            "rescued": {
                'default': 'RESCUE',
            },
            "error": {
                'default': 'ERROR',
            },
            "deleted": {
                'default': 'DELETED',
            },
            "soft-delete": {
                'default': 'SOFT_DELETED',
            },
            "shelved": {
                'default': 'SHELVED',
            },
            "shelved_offloaded": {
                'default': 'SHELVED_OFFLOADED',
            },
        }
        task_map = _STATE_MAP.get(vm_state, dict(default='UNKNOWN'))
        status = task_map.get(task_state, task_map['default'])
        if status == "UNKNOWN":
            LOG.error(("status is UNKNOWN from vm_state=%(vm_state)s "
                    "task_state=%(task_state)s. Bad upgrade or db "
                    "corrupted?"),
                  {'vm_state': vm_state, 'task_state': task_state})
        return status




class GlanceNotificationPayload(object):
    def __init__(self, payload_json, time_format):
        deleted_at = None
        self.images = []
        raw_images = payload_json.get('images', {})
        audit_period_beginning = payload_json.get('audit_period_beginning', "")
        audit_period_ending = payload_json.get('audit_period_ending', "")
        for raw_image in raw_images:
            image = {}
            image['id'] = uuid.uuid4()
            image['resource_id'] = raw_image.get('id', "")
            image['tenant_id'] = payload_json.get('owner', "")
            created_at = raw_image['created_at']
            if raw_image['deleted_at']:
                deleted_at = raw_image['deleted_at']
            image['start_time'] = datetime.datetime.strftime(start_time(
                created_at, audit_period_beginning), time_format)

            image['end_time'] = datetime.datetime.strftime(end_time(
                deleted_at, audit_period_ending), time_format)
            properties = raw_image.get('properties', {})
            image['resource_type'] = properties.get('image_type', "")
            image['server_id'] = properties.get('instance_uuid', "")
            image['server_name'] = properties.get('instance_name', "")
            image['storage'] = raw_image.get('size', "")
            self.images.append(image)



import logging
import pubsubhubbub_publish

import yagi.config
import yagi.handler


with yagi.config.defaults_for('hub') as default:
    default('host', '127.0.0.1')
    default('port', '8000')


LOG = logging.getLogger(__name__)


class PubSubHubBubHandler(yagi.handler.BaseHandler):
    CONFIG_SECTION = "hub"
    AUTO_ACK = True

    def _topic_url(self, key):
        host = yagi.config.get('event_feed', 'feed_host') or '127.0.0.1'
        port = yagi.config.get('event_feed', 'port', default=80)
        if yagi.config.get_bool('event_feed', 'use_https'):
            scheme = 'https'
        else:
            scheme = 'http'
        return '%s://%s:%s/%s' % (scheme, host, port, key)

    def _hub_url(self):
        host = self.config_get('host')
        port = self.config_get('port', default='80')
        scheme = 'https' if self.config_getbool('use_https') else 'http'
        return "%s://%s:%s" % (scheme, host, port)

    def handle_messages(self, messages, env):
        host = self._hub_url()
        topics = {}
        # Compile the list of updated topic urls
        for payload in self.iterate_payloads(messages, env):
            try:
                event_type = payload['event_type']
                if not event_type in topics:
                    topics[event_type] = self._topic_url(event_type)
            except KeyError, e:
                LOG.error('Malformed Notification: %s' % payload)
                LOG.exception(e)

        for event_type, topic in topics.iteritems():
            try:
                LOG.info('Publishing topic %s to %s' % (topic, host))
                pubsubhubbub_publish.publish(host, topic)
            except pubsubhubbub_publish.PublishError, e:
                LOG.exception('Publish failed:\n%s' % e)



import datetime
import logging
import time

import yagi.config
import yagi.filters
import yagi.stats
import yagi.utils


LOG = logging.getLogger(__name__)


class Consumer(object):
    def __init__(self, queue_name, app=None, config=None):
        self.filters = []
        self.queue_name = queue_name
        self.config = yagi.config.config_with("consumer:%s" % queue_name)
        self.connect_time = None
        self.connection = None
        self.consumer = None
        apps = [a.strip() for a in self.config("apps").split(",")]
        prev_app = None
        for a in apps:
            prev_app = yagi.utils.import_class(a)(prev_app,
                                                queue_name=self.queue_name)
        self.app = prev_app
        self.max_messages = int(self.config("max_messages"))

        filter_names = self.config("filters")
        if filter_names:
            filters = (f.strip() for f in filter_names.split(","))
            for f in filters:
                section = yagi.config.config_with("filter:%s" % f)
                map_file = section("map_file")
                method = section("method")
                filter_class = yagi.filters.get_filter(method, map_file, LOG)
                if not filter_class:
                    # Since these should go away, I don't know that it's a big
                    # deal if we can't find one
                    continue
                self.filters.append(filter_class)

    def connect(self, connection, consumer):
        self.disconnect()
        self.connection = connection
        self.consumer = consumer
        self.connect_time = datetime.datetime.now()

    def disconnect(self):
        if self.connection:
            try:
                self.connection.close()
            except Exception, e:
                LOG.info("Error closing broker connection")
                LOG.exception(e)
        self.connection = None
        self.consumer = None

    def idle(self, num_messages, queue_name):
        try:
            self.app.idle(num_messages, queue_name)
        except Exception as e:
            LOG.exception("Error in idle(): \n%s" % e)

    def fetched_messages(self, messages):
        if self.filters:
            env = {'yagi.filters': self.filters}
        else:
            env = {}
        try:
            start_time = time.time()
            self.app(messages, env=env)
            yagi.stats.time_stat(yagi.stats.elapsed_message(),
                                 time.time() - start_time)
        except Exception, e:
            # If we get all the way back out here, that's bad juju
            LOG.exception("Error in fetched_messages: \n%s" % e)

        yagi.stats.increment_stat(yagi.stats.messages_sent(),
                                  len(messages))



import json

import httplib2
import logging

import yagi.config


LOG = logging.getLogger(__name__)


token = None


def no_auth(*args, **kwargs):
    pass


def http_basic_auth(conn, headers, force=False):
    user = yagi.config.get("handler_auth", "user")
    key = yagi.config.get("handler_auth", "key")
    if user and key:
        conn.add_credentials(user, key)


def _rax_auth(conn, headers, force, build_auth_body):
    global token
    if force or not token:
        user = yagi.config.get("handler_auth", "user")
        key = yagi.config.get("handler_auth", "key")
        ssl_check = not (yagi.config.get("handler_auth", "validate_ssl",
            default=False) == "True")
        auth_server = yagi.config.get("handler_auth", "auth_server")
        req = httplib2.Http(disable_ssl_certificate_validation=ssl_check)
        body = build_auth_body(user, key)
        auth_headers = {}
        auth_headers["User-Agent"] = "Yagi"
        auth_headers["Accept"] = "application/json"
        auth_headers["Content-Type"] = "application/json"
        req.follow_all_redirects = True
        LOG.info("Contacting RAX auth server %s" % auth_server)
        LOG.debug("Request for call %r" % auth_headers)
        LOG.debug("Body for call %r" % body)
        res, body = req.request(auth_server, "POST", body=json.dumps(body),
            headers=auth_headers)
        LOG.debug("Respsonse from call %r" % res)
        LOG.debug("Body from call %r" % body)
        # Why 200? it's creating something :-/
        if res.status != 200:
            raise Exception("Authentication failed with HTTP Status %d" %
                            res.status)
        token = json.loads(body)["access"]["token"]["id"]
        LOG.info("Token received: %s" % token)
    headers["X-Auth-Token"] = token


def rax_auth(conn, headers, force=False):
    def _auth_body(user, key):
        return {"auth": {
                "passwordCredentials": {
                    "username": user,
                    "password": key,
                }}}
    _rax_auth(conn, headers, force, _auth_body)


def rax_auth_v2(conn, headers, force=False):
    def _auth_body(user, key):
        return {"auth": {
                "RAX-KSKEY:apiKeyCredentials": {
                    "username": user,
                    "apiKey": key,
                }}}
    _rax_auth(conn, headers, force, _auth_body)


def get_auth_method(method=None):
    if not method:
        method = yagi.config.get("handler_auth", "method")
    if method in globals():
        return globals()[method]
    return None
